# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

"""AST visitor printing out C++"""

import ast
import sys
import tokenize
import warnings

from .formatter import (CppFormatter, format_for_loop, format_literal,
                        format_name_constant,
                        format_reference, write_import, write_import_from)
from .nodedump import debug_format_node
from .qt import ClassFlag, qt_class_flags


def _is_qt_constructor(assign_node):
    """Is this assignment node a plain construction of a Qt class?
       'f = QFile(name)'. Returns the class_name."""
    call = assign_node.value
    if (isinstance(call, ast.Call) and isinstance(call.func, ast.Name)):
        func = call.func.id
        if func.startswith("Q"):
            return func
    return None


def _is_if_main(if_node):
    """Return whether an if statement is: if __name__ == '__main__' """
    test = if_node.test
    return (isinstance(test, ast.Compare)
            and len(test.ops) == 1
            and isinstance(test.ops[0], ast.Eq)
            and isinstance(test.left, ast.Name)
            and test.left.id == "__name__"
            and len(test.comparators) == 1
            and isinstance(test.comparators[0], ast.Constant)
            and test.comparators[0].value == "__main__")


class ConvertVisitor(ast.NodeVisitor, CppFormatter):
    """AST visitor printing out C++
    Note on implementation:
    - Any visit_XXX() overridden function should call self.generic_visit(node)
      to continue visiting
    - When controlling the visiting manually (cf visit_Call()),
      self.visit(child) needs to be called since that dispatches to
      visit_XXX(). This is usually done to prevent undesired output
      for example from references of calls, etc.
    """

    debug = False

    def __init__(self, file_name, output_file):
        ast.NodeVisitor.__init__(self)
        CppFormatter.__init__(self, output_file)
        self._file_name = file_name
        self._class_scope = []  # List of class names
        self._stack = []  # nodes
        self._stack_variables = []  # variables instantiated on stack
        self._debug_indent = 0

    @staticmethod
    def create_ast(filename):
        """Create an Abstract Syntax Tree on which a visitor can be run"""
        node = None
        with tokenize.open(filename) as file:
            node = ast.parse(file.read(), mode="exec")
        return node

    def generic_visit(self, node):
        parent = self._stack[-1] if self._stack else None
        if self.debug:
            self._debug_enter(node, parent)
        self._stack.append(node)
        try:
            super().generic_visit(node)
        except Exception as e:
            line_no = node.lineno if hasattr(node, 'lineno') else -1
            error_message = str(e)
            message = f'{self._file_name}:{line_no}: Error "{error_message}"'
            warnings.warn(message)
            self._output_file.write(f'\n// {error_message}\n')
        del self._stack[-1]
        if self.debug:
            self._debug_leave(node)

    def visit_Add(self, node):
        self._handle_bin_op(node, "+")

    def _is_augmented_assign(self):
        """Is it 'Augmented_assign' (operators +=/-=, etc)?"""
        return self._stack and isinstance(self._stack[-1], ast.AugAssign)

    def visit_AugAssign(self, node):
        """'Augmented_assign', Operators +=/-=, etc."""
        self.INDENT()
        self.generic_visit(node)
        self._output_file.write("\n")

    def visit_Assign(self, node):
        self.INDENT()

        qt_class = _is_qt_constructor(node)
        on_stack = qt_class and qt_class_flags(qt_class) & ClassFlag.INSTANTIATE_ON_STACK

        # Is this a free variable and not a member assignment? Instantiate
        # on stack or give a type
        if len(node.targets) == 1 and isinstance(node.targets[0], ast.Name):
            if qt_class:
                if on_stack:
                    # "QFile f(args)"
                    var = node.targets[0].id
                    self._stack_variables.append(var)
                    self._output_file.write(f"{qt_class} {var}(")
                    self._write_function_args(node.value.args)
                    self._output_file.write(");\n")
                    return
                self._output_file.write("auto *")

        line_no = node.lineno if hasattr(node, 'lineno') else -1
        for target in node.targets:
            if isinstance(target, ast.Tuple):
                w = f"{self._file_name}:{line_no}: List assignment not handled."
                warnings.warn(w)
            elif isinstance(target, ast.Subscript):
                w = f"{self._file_name}:{line_no}: Subscript assignment not handled."
                warnings.warn(w)
            else:
                self._output_file.write(format_reference(target))
                self._output_file.write(' = ')
        if qt_class and not on_stack:
            self._output_file.write("new ")
        self.visit(node.value)
        self._output_file.write(';\n')

    def visit_Attribute(self, node):
        """Format a variable reference (cf visit_Name)"""
        # Default parameter (like Qt::black)?
        if self._ignore_function_def_node(node):
            return
        self._output_file.write(format_reference(node))

    def visit_BinOp(self, node):
        # Parentheses are not exposed, so, every binary operation needs to
        # be enclosed by ().
        self._output_file.write('(')
        self.generic_visit(node)
        self._output_file.write(')')

    def _handle_bin_op(self, node, op):
        """Handle a binary operator which can appear as 'Augmented Assign'."""
        self.generic_visit(node)
        full_op = f" {op}= " if self._is_augmented_assign() else f" {op} "
        self._output_file.write(full_op)

    def visit_BitAnd(self, node):
        self._handle_bin_op(node, "&")

    def visit_BitOr(self, node):
        self._handle_bin_op(node, "|")

    def _format_call(self, node):
        # Decorator list?
        if self._ignore_function_def_node(node):
            return
        f = node.func
        if isinstance(f, ast.Name):
            self._output_file.write(f.id)
        else:
            # Attributes denoting chained calls "a->b()->c()".  Walk along in
            # reverse order, recursing for other calls.
            names = []
            n = f
            while isinstance(n, ast.Attribute):
                names.insert(0, n.attr)
                n = n.value

            if isinstance(n, ast.Name):  # Member or variable reference
                if n.id != "self":
                    sep = "->"
                    if n.id in self._stack_variables:
                        sep = "."
                    elif n.id[0:1].isupper():  # Heuristics for static
                        sep = "::"
                    self._output_file.write(n.id)
                    self._output_file.write(sep)
            elif isinstance(n, ast.Call):  # A preceding call
                self._format_call(n)
                self._output_file.write("->")

            self._output_file.write("->".join(names))

        self._output_file.write('(')
        self._write_function_args(node.args)
        self._output_file.write(')')

    def visit_Call(self, node):
        self._format_call(node)
        # Context manager expression?
        if self._within_context_manager():
            self._output_file.write(";\n")

    def _write_function_args(self, args_node):
        # Manually do visit(), skip the children of func
        for i, arg in enumerate(args_node):
            if i > 0:
                self._output_file.write(', ')
            self.visit(arg)

    def visit_ClassDef(self, node):
        # Manually do visit() to skip over base classes
        # and annotations
        self._class_scope.append(node.name)
        self.write_class_def(node)
        self.indent()
        for b in node.body:
            self.visit(b)
        self.dedent()
        self.indent_line('};')
        del self._class_scope[-1]

    def visit_Div(self, node):
        self._handle_bin_op(node, "/")

    def visit_Eq(self, node):
        self.generic_visit(node)
        self._output_file.write(" == ")

    def visit_Expr(self, node):
        self.INDENT()
        self.generic_visit(node)
        self._output_file.write(';\n')

    def visit_Gt(self, node):
        self.generic_visit(node)
        self._output_file.write(" > ")

    def visit_GtE(self, node):
        self.generic_visit(node)
        self._output_file.write(" >= ")

    def visit_For(self, node):
        # Manually do visit() to get the indentation right.
        # TODO: what about orelse?
        self.indent_line(format_for_loop(node))
        self.indent()
        for b in node.body:
            self.visit(b)
        self.dedent()
        self.indent_line('}')

    def visit_FunctionDef(self, node):
        class_context = self._class_scope[-1] if self._class_scope else None
        for decorator in node.decorator_list:
            func = decorator.func  # (Call)
            if isinstance(func, ast.Name) and func.id == "Slot":
                self._output_file.write("\npublic slots:")
        self.write_function_def(node, class_context)
        # Find stack variables
        for arg in node.args.args:
            if arg.annotation and isinstance(arg.annotation, ast.Name):
                type_name = arg.annotation.id
                flags = qt_class_flags(type_name)
                if flags & ClassFlag.PASS_ON_STACK_MASK:
                    self._stack_variables.append(arg.arg)
        self.indent()
        self.generic_visit(node)
        self.dedent()
        self.indent_line('}')
        self._stack_variables.clear()

    def visit_If(self, node):
        # Manually do visit() to get the indentation right. Note:
        # elsif() is modelled as nested if.

        # Check for the main function
        if _is_if_main(node):
            self._output_file.write("\nint main(int argc, char *argv[])\n{\n")
            self.indent()
            for b in node.body:
                self.visit(b)
            self.indent_string("return 0;\n")
            self.dedent()
            self._output_file.write("}\n")
            return

        self.indent_string('if (')
        self.visit(node.test)
        self._output_file.write(') {\n')
        self.indent()
        for b in node.body:
            self.visit(b)
        self.dedent()
        self.indent_string('}')
        if node.orelse:
            self._output_file.write(' else {\n')
            self.indent()
            for b in node.orelse:
                self.visit(b)
            self.dedent()
            self.indent_string('}')
        self._output_file.write('\n')

    def visit_Import(self, node):
        write_import(self._output_file, node)

    def visit_ImportFrom(self, node):
        write_import_from(self._output_file, node)

    def visit_List(self, node):
        # Manually do visit() to get separators right
        self._output_file.write('{')
        for i, el in enumerate(node.elts):
            if i > 0:
                self._output_file.write(', ')
            self.visit(el)
        self._output_file.write('}')

    def visit_LShift(self, node):
        self.generic_visit(node)
        self._output_file.write(" << ")

    def visit_Lt(self, node):
        self.generic_visit(node)
        self._output_file.write(" < ")

    def visit_LtE(self, node):
        self.generic_visit(node)
        self._output_file.write(" <= ")

    def visit_Mult(self, node):
        self._handle_bin_op(node, "*")

    def _within_context_manager(self):
        """Return whether we are within a context manager (with)."""
        parent = self._stack[-1] if self._stack else None
        return parent and isinstance(parent, ast.withitem)

    def _ignore_function_def_node(self, node):
        """Should this node be ignored within a FunctionDef."""
        if not self._stack:
            return False
        parent = self._stack[-1]
        # A type annotation or default value of an argument?
        if isinstance(parent, (ast.arguments, ast.arg)):
            return True
        if not isinstance(parent, ast.FunctionDef):
            return False
        # Return type annotation or decorator call
        return node == parent.returns or node in parent.decorator_list

    def visit_Index(self, node):
        self._output_file.write("[")
        self.generic_visit(node)
        self._output_file.write("]")

    def visit_Name(self, node):
        """Format a variable reference (cf visit_Attribute)"""
        # Skip Context manager variables, return or argument type annotation
        if self._within_context_manager() or self._ignore_function_def_node(node):
            return
        self._output_file.write(format_reference(node))

    def visit_NameConstant(self, node):
        # Default parameter?
        if self._ignore_function_def_node(node):
            return
        self.generic_visit(node)
        self._output_file.write(format_name_constant(node))

    def visit_Not(self, node):
        self.generic_visit(node)
        self._output_file.write("!")

    def visit_NotEq(self, node):
        self.generic_visit(node)
        self._output_file.write(" != ")

    def visit_Num(self, node):
        self.generic_visit(node)
        self._output_file.write(format_literal(node))

    def visit_RShift(self, node):
        self.generic_visit(node)
        self._output_file.write(" >> ")

    def visit_Return(self, node):
        self.indent_string("return")
        if node.value:
            self._output_file.write(" ")
            self.generic_visit(node)
        self._output_file.write(";\n")

    def visit_Slice(self, node):
        self._output_file.write("[")
        if node.lower:
            self.visit(node.lower)
        self._output_file.write(":")
        if node.upper:
            self.visit(node.upper)
        self._output_file.write("]")

    def visit_Str(self, node):
        self.generic_visit(node)
        self._output_file.write(format_literal(node))

    def visit_Sub(self, node):
        self._handle_bin_op(node, "-")

    def visit_UnOp(self, node):
        self.generic_visit(node)

    def visit_With(self, node):
        self.INDENT()
        self._output_file.write("{ // Converted from context manager\n")
        self.indent()
        for item in node.items:
            self.INDENT()
            if item.optional_vars:
                self._output_file.write(format_reference(item.optional_vars))
                self._output_file.write(" = ")
        self.generic_visit(node)
        self.dedent()
        self.INDENT()
        self._output_file.write("}\n")

    def _debug_enter(self, node, parent=None):
        message = '{}>generic_visit({})'.format('  ' * self ._debug_indent,
                                                debug_format_node(node))
        if parent:
            message += ', parent={}'.format(debug_format_node(parent))
        message += '\n'
        sys.stderr.write(message)
        self._debug_indent += 1

    def _debug_leave(self, node):
        self._debug_indent -= 1
        message = '{}<generic_visit({})\n'.format('  ' * self ._debug_indent,
                                                  type(node).__name__)
        sys.stderr.write(message)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for multiple inheritance'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ObjectType, Point, Str


class SimpleUseCase(ObjectType, Str):
    def __init__(self, name):
        ObjectType.__init__(self)
        Str.__init__(self, name)


class SimpleUseCaseReverse(Str, ObjectType):
    def __init__(self, name):
        ObjectType.__init__(self)
        Str.__init__(self, name)


class SimpleUseCase2(SimpleUseCase):
    def __init__(self, name):
        SimpleUseCase.__init__(self, name)


class ComplexUseCase(SimpleUseCase2, Point):
    def __init__(self, name):
        SimpleUseCase2.__init__(self, name)
        Point.__init__(self)


class ComplexUseCaseReverse(Point, SimpleUseCase2):
    def __init__(self, name):
        SimpleUseCase2.__init__(self, name)
        Point.__init__(self)


class MultipleCppDerivedTest(unittest.TestCase):
    def testInstantiation(self):
        s = SimpleUseCase("Hi")
        self.assertEqual(s, "Hi")
        s.setObjectName(s)
        self.assertEqual(s.objectName(), "Hi")

    def testInstantiation2(self):
        s = SimpleUseCase2("Hi")
        self.assertEqual(s, "Hi")
        s.setObjectName(s)
        self.assertEqual(s.objectName(), "Hi")

    def testComplexInstantiation(self):
        c = ComplexUseCase("Hi")
        self.assertEqual(c, "Hi")
        c.setObjectName(c)
        self.assertEqual(c.objectName(), "Hi")
        c.setX(2)
        self.assertEqual(c.x(), 2)


class MultipleCppDerivedReverseTest(unittest.TestCase):
    def testInstantiation(self):
        s = SimpleUseCaseReverse("Hi")
        self.assertEqual(s, "Hi")
        s.setObjectName(s)
        self.assertEqual(s.objectName(), "Hi")

    def testInstantiation2(self):
        s = SimpleUseCase2("Hi")
        self.assertEqual(s, "Hi")
        s.setObjectName(s)
        self.assertEqual(s.objectName(), "Hi")

    def testComplexInstantiation(self):
        # PYSIDE-1564: This test can no longer work because of this MRO:
        # ('ComplexUseCaseReverse', 'Point', 'SimpleUseCase2', 'SimpleUseCase',
        #  'ObjectType', 'Str', 'Object', 'object')
        # By multiple inheritance Point would be called first but has no argument.
        with self.assertRaises(TypeError):
            c = ComplexUseCaseReverse("Hi")  # noqa: F841
        # c.setObjectName(c)
        # self.assertEqual(c.objectName(), "Hi")
        # c.setX(2);
        # self.assertEqual(c, Point(2, 0))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication
from PySide6.QtGui import QPainter, QLinearGradient, QImage
from PySide6.QtCore import QLine, QLineF, QPoint, QPointF, QRect, QRectF, Qt


try:
    import numpy as np
    HAVE_NUMPY = True
except ModuleNotFoundError:
    HAVE_NUMPY = False


class QPainterDrawText(UsesQApplication):
    def setUp(self):
        super(QPainterDrawText, self).setUp()
        self.image = QImage(32, 32, QImage.Format.Format_ARGB32)
        self.painter = QPainter(self.image)
        self.text = 'teste!'

    def tearDown(self):
        del self.text
        self.painter.end()
        del self.painter
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        super(QPainterDrawText, self).tearDown()

    def testDrawText(self):
        # bug #254
        rect = self.painter.drawText(100, 100, 100, 100,
                                     Qt.AlignmentFlag.AlignCenter | Qt.TextFlag.TextWordWrap,
                                     self.text)
        self.assertTrue(isinstance(rect, QRect))

    def testDrawTextWithRect(self):
        # bug #225
        rect = QRect(100, 100, 100, 100)
        newRect = self.painter.drawText(rect,
                                        Qt.AlignmentFlag.AlignCenter | Qt.TextFlag.TextWordWrap,
                                        self.text)

        self.assertTrue(isinstance(newRect, QRect))

    def testDrawTextWithRectF(self):
        '''QPainter.drawText(QRectF, ... ,QRectF*) inject code'''
        rect = QRectF(100, 52.3, 100, 100)
        newRect = self.painter.drawText(rect,
                                        Qt.AlignmentFlag.AlignCenter | Qt.TextFlag.TextWordWrap,
                                        self.text)

        self.assertTrue(isinstance(newRect, QRectF))

    def testDrawOverloads(self):
        '''Calls QPainter.drawLines overloads, if something is
           wrong Exception and chaos ensues. Bug #395'''
        self.painter.drawLines([QLine(QPoint(0, 0), QPoint(1, 1))])
        self.painter.drawLines([QPoint(0, 0), QPoint(1, 1)])
        self.painter.drawLines([QPointF(0, 0), QPointF(1, 1)])
        self.painter.drawLines([QLineF(QPointF(0, 0), QPointF(1, 1))])
        self.painter.drawPoints([QPoint(0, 0), QPoint(1, 1)])
        self.painter.drawPoints([QPointF(0, 0), QPointF(1, 1)])
        self.painter.drawConvexPolygon([QPointF(10.0, 80.0),
                                        QPointF(20.0, 10.0),
                                        QPointF(80.0, 30.0),
                                        QPointF(90.0, 70.0)])
        self.painter.drawConvexPolygon([QPoint(10.0, 80.0),
                                        QPoint(20.0, 10.0),
                                        QPoint(80.0, 30.0),
                                        QPoint(90.0, 70.0)])
        self.painter.drawPolygon([QPointF(10.0, 80.0),
                                  QPointF(20.0, 10.0),
                                  QPointF(80.0, 30.0),
                                  QPointF(90.0, 70.0)])
        self.painter.drawPolygon([QPoint(10.0, 80.0),
                                  QPoint(20.0, 10.0),
                                  QPoint(80.0, 30.0),
                                  QPoint(90.0, 70.0)])
        self.painter.drawPolyline([QPointF(10.0, 80.0),
                                   QPointF(20.0, 10.0),
                                   QPointF(80.0, 30.0),
                                   QPointF(90.0, 70.0)])
        self.painter.drawPolyline([QPoint(10.0, 80.0),
                                   QPoint(20.0, 10.0),
                                   QPoint(80.0, 30.0),
                                   QPoint(90.0, 70.0)])
        if HAVE_NUMPY:
            x = np.array([10.0, 20.0, 80.0, 90.0])
            y = np.array([80.0, 10.0, 30.0, 70.0])
            self.painter.drawPointsNp(x, y)


class SetBrushWithOtherArgs(UsesQApplication):
    '''Using qpainter.setBrush with args other than QBrush'''

    def testSetBrushGradient(self):
        image = QImage(32, 32, QImage.Format.Format_ARGB32)
        with QPainter(image) as painter:
            gradient = QLinearGradient(0, 0, 0, 0)
            painter.setBrush(gradient)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from mimetypemodel import MimeTypeModel
from PySide6.QtCore import (QDir, QFileInfo, QMimeDatabase, QModelIndex, Qt,
                            Slot)
from PySide6.QtGui import QAction, QKeySequence
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QDialog,
                               QFileDialog, QInputDialog, QMainWindow,
                               QMessageBox, QSplitter, QTextEdit, QTreeView,
                               QWidget)


class MainWindow(QMainWindow):
    def __init__(self, parent: QWidget = None) -> None:
        super().__init__(parent=parent)
        self.m_find_index: int = 0
        self.m_model = MimeTypeModel(self)
        self.m_tree_view = QTreeView(self)
        self.m_details_text = QTextEdit(self)
        self.m_find_matches = []

        self.setWindowTitle("Qt Mime Database Browser")

        # create actions
        self.detect_file_action = QAction(
            "&Detect File Type...", self, shortcut="Ctrl+O", triggered=self.detect_file
        )
        self.exit_action = QAction("E&xit", self, shortcut="Ctrl+Q", triggered=self.close)
        self.m_find_action = QAction("&Find...", self, shortcut="Ctrl+F", triggered=self.find)
        self.m_find_next_action = QAction(
            "Find &Next", self, shortcut="Ctrl+G", triggered=self.find_next
        )
        self.m_find_previous_action = QAction(
            "Find &Previous",
            self,
            shortcut="Ctrl+Shift+G",
            triggered=self.find_previous,
        )
        self.about_action = QAction(
            "About Qt",
            self,
            shortcut=QKeySequence(QKeySequence.StandardKey.HelpContents),
            triggered=QApplication.aboutQt,
        )

        # add action to menu
        self.file_menu = self.menuBar().addMenu("&File")
        self.file_menu.addAction(self.detect_file_action)
        self.file_menu.addAction(self.exit_action)
        self.find_menu = self.menuBar().addMenu("&Edit")
        self.find_menu.addAction(self.m_find_action)
        self.find_menu.addAction(self.m_find_next_action)
        self.find_menu.addAction(self.m_find_previous_action)
        self.about_menu = self.menuBar().addMenu("&About")
        self.about_menu.addAction(self.about_action)

        self.central_splitter = QSplitter(self)
        self.setCentralWidget(self.central_splitter)

        self.m_tree_view.setUniformRowHeights(True)
        self.m_tree_view.setModel(self.m_model)

        self.items = self.m_model.findItems(
            "application/octet-stream",
            Qt.MatchFlag.MatchContains | Qt.MatchFlag.MatchFixedString | Qt.MatchFlag.MatchRecursive
        )

        if self.items:
            self.m_tree_view.expand(self.m_model.indexFromItem(self.items[0]))

        self.m_tree_view.selectionModel().currentChanged.connect(self.current_changed)
        self.central_splitter.addWidget(self.m_tree_view)
        self.m_details_text.setReadOnly(True)
        self.central_splitter.addWidget(self.m_details_text)

        self.update_find_actions()

    @Slot()
    def detect_file(self):
        file_name = QFileDialog.getOpenFileName(self, "Choose File")
        if not file_name:
            return

        mime_database = QMimeDatabase()
        fi = QFileInfo(file_name[0])
        mime_type = mime_database.mimeTypeForFile(fi)
        index = (
            self.m_model.indexForMimeType(mime_type.name())
            if mime_type.isValid()
            else QModelIndex()
        )

        if index.isValid():
            self.statusBar().showMessage(f'{fi.fileName()}" is of type "{mime_type.name()}"')
            self._select_and_goto(index)
        else:
            QMessageBox.information(
                self,
                "Unknown File Type",
                f"The type of {QDir.toNativeSeparators(file_name)} could not be determined.",
            )

    @Slot()
    def find(self):
        input_dialog = QInputDialog(self)
        input_dialog.setWindowTitle("Find")
        input_dialog.setLabelText("Text")
        if input_dialog.exec() != QDialog.DialogCode.Accepted:
            return

        value = input_dialog.textValue().strip()
        if not value:
            return

        self.m_find_matches.clear()
        self.m_find_index = 0
        flags = (Qt.MatchFlag.MatchContains | Qt.MatchFlag.MatchFixedString
                 | Qt.MatchFlag.MatchRecursive)
        items = self.m_model.findItems(value, flags)

        for item in items:
            self.m_find_matches.append(self.m_model.indexFromItem(item))

        self.statusBar().showMessage(f'{len(self.m_find_matches)} mime types match "{value}".')
        self.update_find_actions()

        if self.m_find_matches:
            self._select_and_goto(self.m_find_matches[0])

    @Slot()
    def find_next(self):
        self.m_find_index = self.m_find_index + 1
        if self.m_find_index >= len(self.m_find_matches):
            self.m_find_index = 0
        if self.m_find_index < len(self.m_find_matches):
            self._select_and_goto(self.m_find_matches[self.m_find_index])

    @Slot()
    def find_previous(self):
        self.m_find_index = self.m_find_index - 1
        if self.m_find_index < 0:
            self.m_find_index = len(self.m_find_matches) - 1
        if self.m_find_index >= 0:
            self._select_and_goto(self.m_find_matches[self.m_find_index])

    @Slot(QModelIndex)
    def current_changed(self, index: QModelIndex):
        if index.isValid():
            self.m_details_text.setText(
                MimeTypeModel.formatMimeTypeInfo(self.m_model.mimeType(index))
            )

    def update_find_actions(self):
        self.find_next_previous_enabled = len(self.m_find_matches) > 1
        self.m_find_next_action.setEnabled(self.find_next_previous_enabled)
        self.m_find_previous_action.setEnabled(self.find_next_previous_enabled)

    def _select_and_goto(self, index: QModelIndex):
        self.m_tree_view.scrollTo(index, QAbstractItemView.ScrollHint.PositionAtCenter)
        self.m_tree_view.setCurrentIndex(index)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for finding scope in cases involving inheritance.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import SampleNamespace


class ScopeAndInheritanceTest(unittest.TestCase):
    '''Test cases for finding scope in cases involving inheritance.'''

    def testMethodCorrectlyWrapper(self):
        '''A method returning a type declared in the scope of the method's
        class parent must be found and the method correctly exported.'''
        meth = getattr(SampleNamespace.DerivedFromNamespace,  # noqa: F841
                       'methodReturningTypeFromParentScope')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for the bug #378
http://bugs.openbossa.org/show_bug.cgi?id=378
'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject


class ExtQObject(QObject):
    def __init__(self):
        # "foobar" will become a object attribute that will not be
        # listed on the among the type attributes. Thus for bug
        # condition be correctly triggered the "foobar" attribute
        # must not previously exist in the parent class.
        self.foobar = None
        # The parent __init__ method must be called after the
        # definition of "self.foobar".
        super().__init__()


class TestBug378(unittest.TestCase):
    '''Test case for the bug #378'''

    def testBug378(self):
        obj = ExtQObject()  # noqa: F841


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtGui import QGuiApplication
from PySide6.QtQml import QQmlApplicationEngine

from gradienttexture import GradientTexture  # noqa: F401

from pathlib import Path

import os
import sys

if __name__ == "__main__":
    app = QGuiApplication(sys.argv)
    app.setOrganizationName("QtProject")
    app.setApplicationName("ProceduralTexture")

    engine = QQmlApplicationEngine()
    app_dir = Path(__file__).parent
    engine.addImportPath(os.fspath(app_dir))
    engine.loadFromModule("ProceduralTextureModule", "Main")

    if not engine.rootObjects():
        sys.exit(-1)

    exit_code = app.exec()
    del engine
    sys.exit(exit_code)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QToolbar'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QAction
from PySide6.QtWidgets import QToolBar, QMainWindow

from helper.usesqapplication import UsesQApplication


class AddActionText(UsesQApplication):
    '''Test case for calling QToolbar.addAction passing a text'''

    def setUp(self):
        # Acquire resources
        super(AddActionText, self).setUp()
        self.window = QMainWindow()
        self.toolbar = QToolBar()
        self.window.addToolBar(self.toolbar)

    def tearDown(self):
        # Release resources
        super(AddActionText, self).tearDown()
        del self.toolbar
        del self.window
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testText(self):
        # QToolBar.addAction(text) - add a QToolButton
        self.toolbar.addAction('aaaa')
        self.assertEqual(len(self.toolbar.actions()), 1)
        action = self.toolbar.actions()[0]
        self.assertTrue(isinstance(action, QAction))
        self.assertEqual(action.text(), 'aaaa')


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for overflowing C++ numeric types.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import (Point, doubleLongLong, doubleShort, doubleUnsignedInt,
                    doubleUnsignedLongLong)


class OverflowTest(unittest.TestCase):
    '''Test case for overflowing C++ numeric types.'''

    def assertRaises(self, *args, **kwds):
        if not hasattr(sys, "pypy_version_info"):
            # PYSIDE-535: PyPy complains "Fatal RPython error: NotImplementedError"
            return super().assertRaises(*args, **kwds)

    def testUnsignedInt(self):
        '''C++ function receives an unsigned int argument and raise OverflowError
           if the value is negative.'''
        val = 100
        self.assertEqual(doubleUnsignedInt(val), 2 * val)
        val *= -1
        self.assertRaises(OverflowError, doubleUnsignedInt, val)

    def testLongLong(self):
        '''C++ function receives an long long argument and raise OverflowError
           if the value is negative.'''
        val = 100
        self.assertEqual(doubleLongLong(val), 2 * val)
        val = int(100)
        self.assertEqual(doubleLongLong(val), 2 * val)
        val = (2 << 64) + 1
        self.assertRaises(OverflowError, doubleLongLong, val)

    def testUnsignedLongLong(self):
        '''C++ function receives an unsigned long long argument and raise OverflowError
           if the value is negative.'''
        val = 100
        self.assertEqual(doubleUnsignedLongLong(val), 2 * val)
        val = int(100)
        self.assertEqual(doubleUnsignedLongLong(val), 2 * val)
        val = -100
        self.assertRaises(OverflowError, doubleUnsignedLongLong, val)
        val = int(-200)
        self.assertRaises(OverflowError, doubleUnsignedLongLong, val)

    def testOverflow(self):
        '''Calls function with unsigned int parameter using an overflowing value.'''
        self.assertRaises(OverflowError, doubleUnsignedInt, 42415335332353253)
        doubleUnsignedInt(0xdeadbeef)

    def testShortOverflow(self):
        '''Calls function with short parameter using an overflowing value.'''
        doubleShort(-3)
        self.assertRaises(OverflowError, doubleShort, 0xFFFF * -1)
        self.assertRaises(OverflowError, doubleShort, 0xFFFF + 1)

    def testOverflowOnCtor(self):
        '''Calls object ctor with int parameter using overflowing values.'''
        self.assertRaises(OverflowError, Point, 42415335332353253, 42415335332353253)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from .audio_engine import AudioEngine
from .waveform_item import WaveformItem
from .audio_files_model import AudioFilesModel



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

"""
    Extensive manual test of pyside6-android-deploy

    Note: Not to be added into the CI
"""

import logging
import unittest
import tempfile
import shutil
import sys
import os
import importlib
from pathlib import Path


class TestPySide6Deploy(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pyside_root = Path(__file__).parents[5].resolve()
        example_root = cls.pyside_root / "examples"
        example_application = example_root / "gui" / "analogclock"
        cls.temp_dir = tempfile.mkdtemp()
        cls.temp_example = Path(
            shutil.copytree(example_application, Path(cls.temp_dir) / "analogclock")
        ).resolve()
        cls.current_dir = Path.cwd()

        sys.path.append(str(cls.pyside_root / "sources" / "pyside-tools"))
        cls.deploy_lib = importlib.import_module("deploy_lib")
        cls.android_deploy = importlib.import_module("android_deploy")
        sys.modules["android_deploy"] = cls.android_deploy

        if os.environ.get("WHEEL_PYSIDE") is not None:
            cls.pyside_wheel = Path(os.environ.get("WHEEL_PYSIDE")).resolve()
        else:
            raise Exception("Environment variable WHEEL_PYSIDE does not exist")

        if os.environ.get("WHEEL_SHIBOKEN") is not None:
            cls.shiboken_wheel = Path(os.environ.get("WHEEL_SHIBOKEN")).resolve()
        else:
            raise Exception("Environment variable WHEEL_SHIBOKEN does not exist")

    def setUp(self):
        os.chdir(self.temp_example)
        self.config_file = self.temp_example / "pysidedeploy.spec"

    def testDeployment(self):
        self.android_deploy.main(name="android_app", shiboken_wheel=self.shiboken_wheel,
                                 pyside_wheel=self.pyside_wheel, keep_deployment_files=True,
                                 loglevel=logging.INFO, force=True)

        print("Testing with config file")
        self.android_deploy.main(name="android_app", config_file=self.config_file,
                                 loglevel=logging.INFO, force=True)

    def testWithNdkSdk(self):
        if os.environ.get("ANDROID_SDK_ROOT") is not None:
            android_sdk_root = Path(os.environ.get("ANDROID_SDK_ROOT")).resolve()
        else:
            raise Exception("Environment variable ANDROID_SDK_ROOT does not exist")

        if os.environ.get("ANDROID_NDK_ROOT") is not None:
            android_ndk_root = Path(os.environ.get("ANDROID_NDK_ROOT")).resolve()
        else:
            raise Exception("Environment variable ANDROID_NDK_ROOT does not exist")

        self.android_deploy.main(name="android_app", shiboken_wheel=self.shiboken_wheel,
                                 pyside_wheel=self.pyside_wheel,
                                 ndk_path=android_ndk_root,
                                 sdk_path=android_sdk_root,
                                 keep_deployment_files=True,
                                 loglevel=logging.INFO, force=True)

    def tearDown(self) -> None:
        super().tearDown()
        os.chdir(self.current_dir)

    @classmethod
    def tearDownClass(cls) -> None:
        shutil.rmtree(Path(cls.temp_dir))


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QDate'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QDir, QSettings, QTemporaryDir, QByteArray


class TestQSettings(unittest.TestCase):
    def testConversions(self):
        file = Path(__file__).resolve().parent / 'qsettings_test.ini'
        self.assertTrue(file.is_file())
        file_path = QDir.fromNativeSeparators(os.fspath(file))
        settings = QSettings(file_path, QSettings.Format.IniFormat)

        r = settings.value('var1')
        self.assertEqual(type(r), list)

        r = settings.value('var2')
        self.assertEqual(type(r), str)

        r = settings.value('var2', type=list)
        self.assertEqual(type(r), list)

        # Test mixed conversions
        ba = QByteArray("hello".encode("utf-8"))

        r = settings.value("test", ba, type=QByteArray)
        self.assertEqual(type(r), QByteArray)

        r = settings.value("test", ba, type=str)
        self.assertEqual(type(r), str)

        # Test invalid conversions
        with self.assertRaises(TypeError):
            r = settings.value("test", ba, type=dict)

    def testDefaultValueConversion(self):
        temp_dir = QDir.tempPath()
        dir = QTemporaryDir(f'{temp_dir}/qsettings_XXXXXX')
        self.assertTrue(dir.isValid())
        file_name = dir.filePath('foo.ini')
        settings = QSettings(file_name, QSettings.Format.IniFormat)
        sample_list = ["a", "b"]
        string_list_of_empty = [""]
        settings.setValue('zero_value', 0)
        settings.setValue('empty_list', [])
        settings.setValue('some_strings', sample_list)
        settings.setValue('string_list_of_empty', string_list_of_empty)
        settings.setValue('bool1', False)
        settings.setValue('bool2', True)
        del settings
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

        # Loading values already set
        settings = QSettings(file_name, QSettings.Format.IniFormat)

        # Getting value that doesn't exist
        r = settings.value("variable")
        self.assertEqual(type(r), type(None))

        r = settings.value("variable", type=list)
        self.assertEqual(type(r), list)
        self.assertEqual(len(r), 0)

        # Handling zero value
        r = settings.value('zero_value')
        self.assertEqual(type(r), int)

        r = settings.value('zero_value', type=int)
        self.assertEqual(type(r), int)

        # Empty list
        r = settings.value('empty_list')
        self.assertTrue(len(r) == 0)
        self.assertEqual(type(r), list)

        r = settings.value('empty_list', type=list)
        self.assertTrue(len(r) == 0)
        self.assertEqual(type(r), list)

        r = settings.value('some_strings')
        self.assertEqual(r, sample_list)

        r = settings.value('some_strings', type=list)
        self.assertEqual(r, sample_list)

        r = settings.value('string_list_of_empty', type=list)
        self.assertEqual(r, string_list_of_empty)

        # Booleans
        r = settings.value('bool1')
        self.assertEqual(type(r), bool)

        r = settings.value('bool2')
        self.assertEqual(type(r), bool)

        r = settings.value('bool1', type=bool)
        self.assertEqual(type(r), bool)

        r = settings.value('bool2', type=int)
        self.assertEqual(type(r), int)

        r = settings.value('bool2', type=bool)
        self.assertEqual(type(r), bool)

        # Not set variable, but with default value
        r = settings.value('lala', 22, type=bytes)
        self.assertEqual(type(r), bytes)

        r = settings.value('lala', 22, type=int)
        self.assertEqual(type(r), int)

        r = settings.value('lala', 22, type=float)
        self.assertEqual(type(r), float)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for a class that holds a void pointer.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import VoidHolder, Point
from shiboken6 import Shiboken


class VoidHolderTest(unittest.TestCase):
    '''Test case for void pointer manipulation.'''

    def testGetVoidPointerFromCppAndPutsOnVoidHolder(self):
        '''Passes a void pointer created in C++ to be kept by VoidHolder.'''
        voidptr = VoidHolder.gimmeMeSomeVoidPointer()
        voidholder = VoidHolder(voidptr)
        self.assertEqual(voidptr, voidholder.voidPointer())

    def testPassVoidPointerAsArgument(self):
        '''Passes a void pointer created in C++ as an argument to a function.'''
        voidptr = VoidHolder.gimmeMeSomeVoidPointer()
        voidHolder = VoidHolder()
        returnValue = voidHolder.takeVoidPointer(voidptr)
        self.assertEqual(returnValue, voidptr)

    def testPutRandomObjectInsideVoidHolder(self):
        '''Passes a C++ pointer for an object created in Python to be kept by VoidHolder.'''
        obj = Point(1, 2)
        voidholder = VoidHolder(obj)
        self.assertEqual(Shiboken.getCppPointer(obj)[0], int(voidholder.voidPointer()))

    def testGetNoneObjectFromVoidHolder(self):
        '''A VoidHolder created without parameters returns a NULL pointer
           that should be converted to a Python None.'''
        voidholder = VoidHolder()
        self.assertEqual(voidholder.voidPointer(), None)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QByteArray
from helper.docmodifier import DocModifier


class BaseQByteArrayOperatorIAdd:
    '''Base class for QByteArray += operator tests.

    Implementing classes should inherit from unittest.TestCase and implement
    setUp, setting self.obj and self.orig_obj to the target QByteArray and original
    one, respectively'''

    __metaclass__ = DocModifier

    def testSingleString(self):
        '''QByteArray += bytes of size 1'''
        s = bytes('0', "UTF-8")
        self.obj += s
        self.assertEqual(self.obj, self.orig_obj + s)
        self.assertEqual(self.obj.size(), self.orig_obj.size() + len(s))

    def testString(self):
        '''QByteArray += bytes of size > 1'''
        s = bytearray(bytes('dummy', "UTF-8"))
        self.obj += s  # XXx iadd support abytearray
        self.assertEqual(self.obj, self.orig_obj + s)
        self.assertEqual(self.obj.size(), self.orig_obj.size() + len(s))

    def testQByteArray(self):
        '''QByteArray += QByteArray'''
        s = QByteArray(bytes('array', "UTF-8"))
        self.obj += s
        self.assertEqual(self.obj, self.orig_obj + s)


class NullQByteArrayOperatorIAdd(unittest.TestCase, BaseQByteArrayOperatorIAdd):
    '''Test case for operator QByteArray += on null QByteArrays'''

    doc_prefix = 'Null object'
    doc_filter = lambda x: x.startswith('test')  # noqa: E731

    def setUp(self):
        self.obj = QByteArray()
        self.orig_obj = QByteArray()


class ValidQByteArrayOperatorIAdd(unittest.TestCase, BaseQByteArrayOperatorIAdd):
    '''Test case for operator QByteArray += on valid QByteArrays'''

    doc_prefix = 'Valid object'
    doc_filter = lambda x: x.startswith('test')  # noqa: E731

    def setUp(self):
        self.obj = QByteArray(bytes('some byte array', "UTF-8"))
        self.orig_obj = QByteArray(bytes('some byte array', "UTF-8"))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QByteArray operators'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QByteArray


class QByteArrayOperatorEqual(unittest.TestCase):
    '''TestCase for operator QByteArray == QByteArray'''

    def testDefault(self):
        # QByteArray() == QByteArray()
        obj1 = QByteArray()
        obj2 = QByteArray()
        self.assertEqual(obj1, obj2)

    def testSimple(self):
        # QByteArray(some_string) == QByteArray(some_string)
        string = bytes('egg snakes', "UTF-8")
        self.assertEqual(QByteArray(string), QByteArray(string))

    def testPyString(self):
        # QByteArray(string) == string
        string = bytes('my test string', "UTF-8")
        self.assertEqual(QByteArray(string), string)


class QByteArrayOperatorAt(unittest.TestCase):
    '''TestCase for operator QByteArray[]'''

    def testInRange(self):
        # QByteArray[x] where x is a valid index
        string = 'abcdefgh'
        obj = QByteArray(bytes(string, "UTF-8"))

        for i in range(len(string)):
            self.assertEqual(obj[i], bytes(string[i], "UTF-8"))

    def testInRangeReverse(self):
        # QByteArray[x] where x is a valid index (reverse order)
        string = 'abcdefgh'
        obj = QByteArray(bytes(string, "UTF-8"))

        for i in range(len(string) - 1, 0, -1):
            self.assertEqual(obj[i], bytes(string[i], "UTF-8"))

    def testOutOfRange(self):
        # QByteArray[x] where x is out of index
        string = bytes('1234567', "UTF-8")
        obj = QByteArray(string)
        self.assertRaises(IndexError, lambda: obj[len(string)])

    def testNullStrings(self):
        ba = QByteArray(bytes('\x00', "UTF-8"))
        self.assertEqual(ba.at(0), '\x00')
        self.assertEqual(ba[0], bytes('\x00', "UTF-8"))


class QByteArrayOperatorLen(unittest.TestCase):
    '''Test case for __len__ operator of QByteArray'''

    def testBasic(self):
        '''QByteArray __len__'''
        self.assertEqual(len(QByteArray()), 0)
        self.assertEqual(len(QByteArray(bytes('', "UTF-8"))), 0)
        self.assertEqual(len(QByteArray(bytes(' ', "UTF-8"))), 1)
        self.assertEqual(len(QByteArray(bytes('yabadaba', "UTF-8"))), 8)


class QByteArrayOperatorStr(unittest.TestCase):
    '''Test case for __str__ operator of QByteArray'''

    def testBasic(self):
        '''QByteArray __str__'''
        self.assertEqual(QByteArray().__str__(), str(b''))
        self.assertEqual(QByteArray(bytes('', "UTF-8")).__str__(), str(b''))
        self.assertEqual(QByteArray(bytes('aaa', "UTF-8")).__str__(), str(b'aaa'))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QTextEdit and ownership problems.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QTextEdit

from helper.usesqapplication import UsesQApplication


class DontTouchReference(UsesQApplication):
    '''Check if the QTextTable returned by QTextCursor.insertTable() is not
    referenced by the QTextCursor that returns it.'''

    def setUp(self):
        super(DontTouchReference, self).setUp()
        self.editor = QTextEdit()
        self.cursor = self.editor.textCursor()
        self.table = self.cursor.insertTable(1, 1)

    @unittest.skipUnless(hasattr(sys, "getrefcount"), f"{sys.implementation.name} has no refcount")
    def testQTextTable(self):
        # methods which return QTextTable should not increment its reference
        self.assertEqual(sys.getrefcount(self.table), 2)
        f = self.cursor.currentFrame()
        del f
        self.assertEqual(sys.getrefcount(self.table), 2)
        # destroying the cursor should not raise any "RuntimeError: internal
        # C++ object already deleted." when accessing the QTextTable
        del self.cursor
        self.assertEqual(sys.getrefcount(self.table), 2)
        cell = self.table.cellAt(0, 0)  # noqa: F841


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Wrapper validity tests for arguments.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Polygon, Point


class WrapperValidityOfArgumentsTest(unittest.TestCase):
    '''Wrapper validity tests for arguments.'''

    def testInvalidArgumentToMethod(self):
        '''Call to method using invalidated Python wrapper as argument should raise RuntimeError.'''
        poly = Polygon()
        Polygon.stealOwnershipFromPython(poly)
        self.assertRaises(RuntimeError, Polygon.doublePolygonScale, poly)

    def testInvalidArgumentToConstructor(self):
        '''Call to constructor using invalidated Python wrapper as argument
           should raise RuntimeError.'''
        pt = Point(1, 2)
        Polygon.stealOwnershipFromPython(pt)
        self.assertRaises(RuntimeError, Polygon, pt)

    def testInvalidArgumentWithImplicitConversion(self):
        '''Call to method using invalidated Python wrapper to be implicitly converted
           should raise RuntimeError.'''
        pt = Point(1, 2)
        Polygon.stealOwnershipFromPython(pt)
        self.assertRaises(RuntimeError, Polygon.doublePolygonScale, pt)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QMimeDatabase, QMimeType, QModelIndex, QObject, Qt, qWarning
from PySide6.QtGui import QStandardItem, QStandardItemModel

mimeTypeRole = Qt.ItemDataRole.UserRole + 1
iconQueriedRole = Qt.ItemDataRole.UserRole + 2


def createRow(t: QMimeType):
    name_item = QStandardItem(t.name())
    flags = Qt.ItemFlag.ItemIsSelectable | Qt.ItemFlag.ItemIsEnabled
    name_item.setData(t, mimeTypeRole)
    name_item.setData(False, iconQueriedRole)
    name_item.setFlags(flags)
    name_item.setToolTip(t.comment())
    return [name_item]


class MimeTypeModel(QStandardItemModel):
    def __init__(self, parent: QObject = None):
        super().__init__(0, 1, parent)
        self.setHorizontalHeaderLabels(["Name"])
        self.m_name_index_hash = {}
        self.populate()

    def populate(self):
        mime_database = QMimeDatabase()
        all_types: list[QMimeType] = mime_database.allMimeTypes()

        # Move top level types to rear end of list, sort this partition,
        # create top level items and truncate the list.
        with_parent_mimetypes, without_parent_mimetypes = [], []

        for mime_type in all_types:
            if mime_type.parentMimeTypes():
                with_parent_mimetypes.append(mime_type)
            else:
                without_parent_mimetypes.append(mime_type)

        without_parent_mimetypes.sort(key=lambda x: x.name())

        for top_level_type in without_parent_mimetypes:
            row = createRow(top_level_type)
            self.appendRow(row)
            self.m_name_index_hash[top_level_type.name()] = self.indexFromItem(row[0])

        all_types = with_parent_mimetypes

        while all_types:
            # Find a type inheriting one that is already in the model.
            name_index_value: QModelIndex = None
            name_index_key = ""
            for mime_type in all_types:
                name_index_value = self.m_name_index_hash.get(
                    mime_type.parentMimeTypes()[0]
                )
                if name_index_value:
                    name_index_key = mime_type.parentMimeTypes()[0]
                    break

            if not name_index_value:
                orphaned_mime_types = ", ".join(
                    [mime_type.name() for mime_type in all_types]
                )
                qWarning(f"Orphaned mime types: {orphaned_mime_types}")
                break

            # Move types inheriting the parent type to rear end of list, sort this partition,
            # append the items to parent and truncate the list.
            parent_name = name_index_key
            with_parent_name, without_parent_name = [], []

            for mime_type in all_types:
                if parent_name in mime_type.parentMimeTypes():
                    with_parent_name.append(mime_type)
                else:
                    without_parent_name.append(mime_type)

            without_parent_name.sort(key=lambda x: x.name())
            parent_item = self.itemFromIndex(name_index_value)

            for mime_type in with_parent_name:
                row = createRow(mime_type)
                parent_item.appendRow(row)
                self.m_name_index_hash[mime_type.name()] = self.indexFromItem(row[0])

            all_types = without_parent_name

    def mimeType(self, index: QModelIndex):
        return index.data(mimeTypeRole)

    def indexForMimeType(self, name):
        return self.m_name_index_hash[name]

    @staticmethod
    def formatMimeTypeInfo(t: QMimeType):
        out = f"<html><head/><body><h3><center>{t.name()}</center></h3><br><table>"
        if aliases_str := ", ".join(t.aliases()):
            out += f"<tr><td>Aliases:</td><td> ({aliases_str})"

        out += (
            f"</td></tr><tr><td>Comment:</td><td>{t.comment()}"
            f"</td></tr><tr><td>Icon name:</td><td>{t.iconName()}</td></tr>"
            f"<tr><td>Generic icon name</td><td>{t.genericIconName()}</td></tr>"
        )

        if filter_str := t.filterString():
            out += f"<tr><td>Filter:</td><td>{filter_str}</td></tr>"

        if patterns_str := ", ".join(t.globPatterns()):
            out += f"<tr><td>Glob patterns:</td><td>{patterns_str}</td></tr>"

        if parentMimeTypes_str := ", ".join(t.parentMimeTypes()):
            out += f"<tr><td>Parent types:</td><td>{parentMimeTypes_str}</td></tr>"

        if suffixes := t.suffixes():
            out += "<tr><td>Suffixes:</td><td>"
            if preferredSuffix := t.preferredSuffix():
                suffixes.remove(preferredSuffix)
                out += f"<b>{preferredSuffix}</b> "
            suffixes_str = ", ".join(suffixes)
            out += f"{suffixes_str}</td></tr>"

        out += "</table></body></html>"

        return out



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QImage'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QImage
from helper.usesqapplication import UsesQApplication
from xpm_data import xpm


class QImageTest(UsesQApplication):
    '''Test case for calling setPixel with float as argument'''

    def testQImageStringBuffer(self):
        '''Test if the QImage signatures receiving string buffers exist.'''
        file = Path(__file__).resolve().parent / 'sample.png'
        self.assertTrue(file.is_file())
        img0 = QImage(file)

        # btw let's test the bits() method
        img1 = QImage(img0.bits(), img0.width(), img0.height(), img0.format())
        img1.setColorSpace(img0.colorSpace())
        self.assertEqual(img0, img1)
        img2 = QImage(img0.bits(), img0.width(), img0.height(), img0.bytesPerLine(), img0.format())
        img2.setColorSpace(img0.colorSpace())
        self.assertEqual(img0, img2)

        # test scanLine method
        data1 = img0.scanLine(0)
        data2 = img1.scanLine(0)
        self.assertEqual(data1, data2)

    def testEmptyBuffer(self):
        img = QImage(bytes('', "UTF-8"), 100, 100, QImage.Format.Format_ARGB32)  # noqa: F841

    def testEmptyStringAsBuffer(self):
        img = QImage(bytes('', "UTF-8"), 100, 100, QImage.Format.Format_ARGB32)  # noqa: F841

    def testXpmConstructor(self):
        img = QImage(xpm)
        self.assertFalse(img.isNull())
        self.assertEqual(img.width(), 27)
        self.assertEqual(img.height(), 22)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import (QByteArray, QEvent, QObject, QPoint, Qt, QTimer,
                            QXmlStreamReader, Slot)
from PySide6.QtGui import QPalette
from PySide6.QtNetwork import (QNetworkAccessManager, QNetworkReply,
                               QNetworkRequest)
from PySide6.QtWidgets import QAbstractItemView, QFrame, QTreeWidget, QTreeWidgetItem


class GSuggestCompletion(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.editor = parent
        self.popup = QTreeWidget()
        self.popup.setWindowFlags(Qt.WindowType.Popup)
        self.popup.setFocusPolicy(Qt.FocusPolicy.NoFocus)
        self.popup.setFocusProxy(parent)
        self.popup.setMouseTracking(True)

        self.popup.setColumnCount(1)
        self.popup.setUniformRowHeights(True)
        self.popup.setRootIsDecorated(False)
        self.popup.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        self.popup.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.popup.setFrameStyle(QFrame.Shape.Box | QFrame.Shadow.Plain)
        self.popup.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.popup.header().hide()

        self.popup.installEventFilter(self)

        self.popup.itemClicked.connect(self.done_completion)

        self.timer = QTimer()
        self.timer.setSingleShot(True)
        self.timer.setInterval(500)
        self.timer.timeout.connect(self.auto_suggest)
        self.editor.textEdited.connect(self.timer.start)

        self.network_manager = QNetworkAccessManager()
        self.network_manager.finished.connect(self.handle_network_data)

    def eventFilter(self, obj: QObject, ev: QEvent):
        if obj is not self.popup:
            return False
        if ev.type() == QEvent.Type.MouseButtonPress:
            self.popup.hide()
            self.editor.setFocus()
            return True

        if ev.type() == QEvent.Type.KeyPress:
            consumed = False
            key = ev.key()
            if key in (Qt.Key.Key_Enter, Qt.Key.Key_Return):
                self.done_completion()
                consumed = True
            elif key == Qt.Key.Key_Escape:
                self.editor.setFocus()
                self.popup.hide()
                consumed = True
            elif key in (
                Qt.Key.Key_Up,
                Qt.Key.Key_Down,
                Qt.Key.Key_Home,
                Qt.Key.Key_End,
                Qt.Key.Key_PageUp,
                Qt.Key.Key_PageDown,
            ):
                pass
            else:
                self.editor.setFocus()
                self.editor.event(ev)
                self.popup.hide()
            return consumed
        return False

    def show_completion(self, choices: list[str]):
        if not choices:
            return
        pal = self.editor.palette()
        color = pal.color(QPalette.ColorGroup.Disabled, QPalette.ColorRole.WindowText)

        self.popup.setUpdatesEnabled(False)
        self.popup.clear()

        for choice in choices:
            item = QTreeWidgetItem(self.popup)
            item.setText(0, choice)
            item.setForeground(0, color)

        self.popup.setCurrentItem(self.popup.topLevelItem(0))
        self.popup.resizeColumnToContents(0)
        self.popup.setUpdatesEnabled(True)

        self.popup.move(self.editor.mapToGlobal(QPoint(0, self.editor.height())))
        self.popup.setFocus()
        self.popup.show()

    @Slot()
    def done_completion(self):
        self.timer.stop()
        self.popup.hide()
        self.editor.setFocus()
        if item := self.popup.currentItem():
            self.editor.setText(item.text(0))
            self.editor.returnPressed.emit()

    @Slot()
    def auto_suggest(self):
        s = self.editor.text()
        url = f"https://google.com/complete/search?output=toolbar&q={s}"
        self.network_manager.get(QNetworkRequest(url))

    def prevent_suggest(self):
        self.timer.stop()

    @Slot(QNetworkReply)
    def handle_network_data(self, network_reply: QNetworkReply):
        if network_reply.error() == QNetworkReply.NetworkError.NoError:
            choices: list[str] = []

            response: QByteArray = network_reply.readAll()
            xml = QXmlStreamReader(str(response))
            while not xml.atEnd():
                xml.readNext()
                if xml.tokenType() == QXmlStreamReader.TokenType.StartElement:
                    if xml.name() == "suggestion":
                        s = xml.attributes().value("data")
                        choices.append(s)
            self.show_completion(choices)

        network_reply.deleteLater()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Slot
from PySide6.QtGui import QDesktopServices
from PySide6.QtWidgets import QLineEdit

from googlesuggest import GSuggestCompletion


class SearchBox(QLineEdit):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.completer = GSuggestCompletion(self)

        self.returnPressed.connect(self.do_search)
        self.setWindowTitle("Search with Google")

        self.adjustSize()
        self.resize(400, self.height())
        self.setFocus()

    @Slot()
    def do_search(self):
        self.completer.prevent_suggest()
        url = f"https://www.google.com/search?q={self.text()}"
        QDesktopServices.openUrl(url)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from helper.helper import qmlcomponent_errorstring  # noqa: E402
from helper.timedqguiapplication import TimedQGuiApplication  # noqa: E402

from PySide6.QtCore import QObject, QUrl, qVersion  # noqa: E402
from PySide6.QtGui import QRasterWindow  # noqa: 402
from PySide6.QtQml import QmlNamedElement, QmlForeign, QQmlEngine, QQmlComponent  # noqa: E402


"""Test the QmlForeign decorator, letting the QQmlEngine create a QRasterWindow."""


QML_IMPORT_NAME = "Foreign"
QML_IMPORT_MAJOR_VERSION = 1


@QmlNamedElement("QRasterWindow")
@QmlForeign(QRasterWindow)
class RasterWindowForeign(QObject):
    def __init__(self, parent=None):
        super().__init__(parent)


class TestQmlForeign(TimedQGuiApplication):

    def testIt(self):
        engine = QQmlEngine()
        file = Path(__file__).resolve().parent / 'registerforeign.qml'
        self.assertTrue(file.is_file())
        component = QQmlComponent(engine, QUrl.fromLocalFile(file))
        window = component.create()
        self.assertTrue(window, qmlcomponent_errorstring(component))
        self.assertEqual(type(window), QRasterWindow)
        window.setTitle(f"Qt {qVersion()}")
        window.show()
        self.app.exec()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtCore import (QDateTime, QDir, QLibraryInfo, QSysInfo, Qt,
                            QTimer, Slot, qVersion)
from PySide6.QtGui import (QCursor, QDesktopServices, QGuiApplication, QIcon,
                           QKeySequence, QShortcut, QStandardItem,
                           QStandardItemModel)
from PySide6.QtWidgets import (QApplication, QCheckBox, QComboBox,
                               QCommandLinkButton, QDateTimeEdit, QDial,
                               QDialog, QDialogButtonBox, QFileSystemModel,
                               QGridLayout, QGroupBox, QHBoxLayout, QLabel,
                               QLineEdit, QListView, QMenu, QPlainTextEdit,
                               QProgressBar, QPushButton, QRadioButton,
                               QScrollBar, QSizePolicy, QSlider, QSpinBox,
                               QStyleFactory, QTableWidget, QTabWidget,
                               QTextBrowser, QTextEdit, QToolBox, QToolButton,
                               QTreeView, QVBoxLayout, QWidget)

POEM = """Twinkle, twinkle, little star,
How I wonder what you are.
Up above the world so high,
Like a diamond in the sky.
Twinkle, twinkle, little star,
How I wonder what you arenot"""

DIR_OPEN_ICON = ":/qt-project.org/styles/commonstyle/images/diropen-128.png"

COMPUTER_ICON = ":/qt-project.org/styles/commonstyle/images/computer-32.png"

SYSTEMINFO = """<html><head/><body>
<h3>Python</h3><p>{}</p>
<h3>Qt Build</h3><p>{}</p>
<h3>Operating System</h3><p>"{}" / {}</p>
<h3>Screens</h3>
{}
</body></html>"""


def class_name(o):
    return o.metaObject().className()


def help_url(page):
    """Build a Qt help URL from the page name"""
    major_version = qVersion().split('.')[0]
    return f"https://doc.qt.io/qt-{major_version}/{page}.html"


def launch_help(widget):
    """Launch a widget's help page"""
    url = help_url(class_name(widget).lower())
    QDesktopServices.openUrl(url)


def launch_module_help():
    QDesktopServices.openUrl(help_url("qtwidgets-index"))


def init_widget(w, name):
    """Init a widget for the gallery, give it a tooltip showing the
       class name"""
    w.setObjectName(name)
    w.setToolTip(class_name(w))


def style_names():
    """Return a list of styles, default platform style first"""
    default_style_name = QApplication.style().objectName().lower()
    result = []
    for style in QStyleFactory.keys():
        if style.lower() == default_style_name:
            result.insert(0, style)
        else:
            result.append(style)
    return result


def embed_into_hbox_layout(w, margin=5):
    """Embed a widget into a layout to give it a frame"""
    result = QWidget()
    layout = QHBoxLayout(result)
    layout.setContentsMargins(margin, margin, margin, margin)
    layout.addWidget(w)
    return result


def format_geometry(rect):
    """Format a geometry as a X11 geometry specification"""
    w = rect.width()
    h = rect.height()
    x = rect.x()
    y = rect.y()
    return f"{w}x{h}{x:+d}{y:+d}"


def screen_info(widget):
    """Format information on the screens"""
    policy = QGuiApplication.highDpiScaleFactorRoundingPolicy()
    policy_string = str(policy).split('.')[-1]
    result = f"<p>High DPI scale factor rounding policy: {policy_string}</p><ol>"
    for screen in QGuiApplication.screens():
        current = screen == widget.screen()
        result += "<li>"
        if current:
            result += "<i>"
        name = screen.name()
        geometry = format_geometry(screen.geometry())
        dpi = int(screen.logicalDotsPerInchX())
        dpr = screen.devicePixelRatio()
        result += f'"{name}" {geometry} {dpi}DPI, DPR={dpr}'
        if current:
            result += "</i>"
        result += "</li>"
    result += "</ol>"
    return result


class WidgetGallery(QDialog):
    """Dialog displaying a gallery of Qt Widgets"""

    def __init__(self):
        super().__init__()

        self.setWindowIcon(QIcon(':/qt-project.org/logos/pysidelogo.png'))
        self._progress_bar = self.create_progress_bar()

        self._style_combobox = QComboBox()
        init_widget(self._style_combobox, "styleComboBox")
        self._style_combobox.addItems(style_names())

        style_label = QLabel("Style:")
        init_widget(style_label, "style_label")
        style_label.setBuddy(self._style_combobox)

        help_label = QLabel("Press F1 over a widget to see Documentation")
        init_widget(help_label, "help_label")

        disable_widgets_checkbox = QCheckBox("Disable widgets")
        init_widget(disable_widgets_checkbox, "disable_widgets_checkbox")

        buttons_groupbox = self.create_buttons_groupbox()
        itemview_tabwidget = self.create_itemview_tabwidget()
        simple_input_widgets_groupbox = self.create_simple_inputwidgets_groupbox()
        text_toolbox = self.create_text_toolbox()

        self._style_combobox.textActivated.connect(self.change_style)
        disable_widgets_checkbox.toggled.connect(buttons_groupbox.setDisabled)
        disable_widgets_checkbox.toggled.connect(text_toolbox.setDisabled)
        disable_widgets_checkbox.toggled.connect(itemview_tabwidget.setDisabled)
        disable_widgets_checkbox.toggled.connect(simple_input_widgets_groupbox.setDisabled)

        help_shortcut = QShortcut(self)
        help_shortcut.setKey(QKeySequence.StandardKey.HelpContents)
        help_shortcut.activated.connect(self.help_on_current_widget)

        top_layout = QHBoxLayout()
        top_layout.addWidget(style_label)
        top_layout.addWidget(self._style_combobox)
        top_layout.addStretch(1)
        top_layout.addWidget(help_label)
        top_layout.addStretch(1)
        top_layout.addWidget(disable_widgets_checkbox)

        dialog_buttonbox = QDialogButtonBox(QDialogButtonBox.StandardButton.Help
                                            | QDialogButtonBox.StandardButton.Close)
        init_widget(dialog_buttonbox, "dialogButtonBox")
        dialog_buttonbox.helpRequested.connect(launch_module_help)
        dialog_buttonbox.rejected.connect(self.reject)

        main_layout = QGridLayout(self)
        main_layout.addLayout(top_layout, 0, 0, 1, 2)
        main_layout.addWidget(buttons_groupbox, 1, 0)
        main_layout.addWidget(simple_input_widgets_groupbox, 1, 1)
        main_layout.addWidget(itemview_tabwidget, 2, 0)
        main_layout.addWidget(text_toolbox, 2, 1)
        main_layout.addWidget(self._progress_bar, 3, 0, 1, 2)
        main_layout.addWidget(dialog_buttonbox, 4, 0, 1, 2)

        qv = qVersion()
        self.setWindowTitle(f"Widget Gallery Qt {qv}")

    def setVisible(self, visible):
        super(WidgetGallery, self).setVisible(visible)
        if visible:
            self.windowHandle().screenChanged.connect(self.update_systeminfo)
            self.update_systeminfo()

    @Slot(str)
    def change_style(self, style_name):
        QApplication.setStyle(QStyleFactory.create(style_name))

    @Slot()
    def advance_progressbar(self):
        cur_val = self._progress_bar.value()
        max_val = self._progress_bar.maximum()
        self._progress_bar.setValue(cur_val + (max_val - cur_val) / 100)

    def create_buttons_groupbox(self):
        result = QGroupBox("Buttons")
        init_widget(result, "buttons_groupbox")

        default_pushbutton = QPushButton("Default Push Button")
        init_widget(default_pushbutton, "default_pushbutton")
        default_pushbutton.setDefault(True)

        toggle_pushbutton = QPushButton("Toggle Push Button")
        init_widget(toggle_pushbutton, "toggle_pushbutton")
        toggle_pushbutton.setCheckable(True)
        toggle_pushbutton.setChecked(True)

        flat_pushbutton = QPushButton("Flat Push Button")
        init_widget(flat_pushbutton, "flat_pushbutton")
        flat_pushbutton.setFlat(True)

        toolbutton = QToolButton()
        init_widget(toolbutton, "toolButton")
        toolbutton.setText("Tool Button")

        menu_toolbutton = QToolButton()
        init_widget(menu_toolbutton, "menuButton")
        menu_toolbutton.setText("Menu Button")
        tool_menu = QMenu(menu_toolbutton)
        menu_toolbutton.setPopupMode(QToolButton.ToolButtonPopupMode.InstantPopup)
        tool_menu.addAction("Option")
        tool_menu.addSeparator()
        action = tool_menu.addAction("Checkable Option")
        action.setCheckable(True)
        menu_toolbutton.setMenu(tool_menu)
        tool_layout = QHBoxLayout()
        tool_layout.addWidget(toolbutton)
        tool_layout.addWidget(menu_toolbutton)

        commandlinkbutton = QCommandLinkButton("Command Link Button")
        init_widget(commandlinkbutton, "commandLinkButton")
        commandlinkbutton.setDescription("Description")

        button_layout = QVBoxLayout()
        button_layout.addWidget(default_pushbutton)
        button_layout.addWidget(toggle_pushbutton)
        button_layout.addWidget(flat_pushbutton)
        button_layout.addLayout(tool_layout)
        button_layout.addWidget(commandlinkbutton)
        button_layout.addStretch(1)

        radiobutton_1 = QRadioButton("Radio button 1")
        init_widget(radiobutton_1, "radioButton1")
        radiobutton_2 = QRadioButton("Radio button 2")
        init_widget(radiobutton_2, "radioButton2")
        radiobutton_3 = QRadioButton("Radio button 3")
        init_widget(radiobutton_3, "radioButton3")
        radiobutton_1.setChecked(True)

        checkbox = QCheckBox("Tri-state check box")
        init_widget(checkbox, "checkBox")
        checkbox.setTristate(True)
        checkbox.setCheckState(Qt.CheckState.PartiallyChecked)

        checkable_layout = QVBoxLayout()
        checkable_layout.addWidget(radiobutton_1)
        checkable_layout.addWidget(radiobutton_2)
        checkable_layout.addWidget(radiobutton_3)
        checkable_layout.addWidget(checkbox)
        checkable_layout.addStretch(1)

        main_layout = QHBoxLayout(result)
        main_layout.addLayout(button_layout)
        main_layout.addLayout(checkable_layout)
        main_layout.addStretch()
        return result

    def create_text_toolbox(self):
        result = QToolBox()
        init_widget(result, "toolBox")

        # Create centered/italic HTML rich text
        rich_text = "<html><head/><body><i>"
        for line in POEM.split('\n'):
            rich_text += f"<center>{line}</center>"
        rich_text += "</i></body></html>"

        text_edit = QTextEdit(rich_text)
        init_widget(text_edit, "textEdit")
        plain_textedit = QPlainTextEdit(POEM)
        init_widget(plain_textedit, "plainTextEdit")

        self._systeminfo_textbrowser = QTextBrowser()
        init_widget(self._systeminfo_textbrowser, "systemInfoTextBrowser")

        result.addItem(embed_into_hbox_layout(text_edit), "Text Edit")
        result.addItem(embed_into_hbox_layout(plain_textedit),
                       "Plain Text Edit")
        result.addItem(embed_into_hbox_layout(self._systeminfo_textbrowser),
                       "Text Browser")
        return result

    def create_itemview_tabwidget(self):
        result = QTabWidget()
        init_widget(result, "bottomLeftTabWidget")
        result.setSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Ignored)

        tree_view = QTreeView()
        init_widget(tree_view, "treeView")
        filesystem_model = QFileSystemModel(tree_view)
        filesystem_model.setRootPath(QDir.rootPath())
        tree_view.setModel(filesystem_model)

        table_widget = QTableWidget()
        init_widget(table_widget, "tableWidget")
        table_widget.setRowCount(10)
        table_widget.setColumnCount(10)

        list_model = QStandardItemModel(0, 1, result)

        list_model.appendRow(QStandardItem(QIcon(DIR_OPEN_ICON), "Directory"))
        list_model.appendRow(QStandardItem(QIcon(COMPUTER_ICON), "Computer"))

        list_view = QListView()
        init_widget(list_view, "listView")
        list_view.setModel(list_model)

        icon_mode_listview = QListView()
        init_widget(icon_mode_listview, "iconModeListView")

        icon_mode_listview.setViewMode(QListView.ViewMode.IconMode)
        icon_mode_listview.setModel(list_model)

        result.addTab(embed_into_hbox_layout(tree_view), "Tree View")
        result.addTab(embed_into_hbox_layout(table_widget), "Table")
        result.addTab(embed_into_hbox_layout(list_view), "List")
        result.addTab(embed_into_hbox_layout(icon_mode_listview),
                      "Icon Mode List")
        return result

    def create_simple_inputwidgets_groupbox(self):
        result = QGroupBox("Simple Input Widgets")
        init_widget(result, "bottomRightGroupBox")
        result.setCheckable(True)
        result.setChecked(True)

        lineedit = QLineEdit("s3cRe7")
        init_widget(lineedit, "lineEdit")
        lineedit.setClearButtonEnabled(True)
        lineedit.setEchoMode(QLineEdit.EchoMode.Password)

        spin_box = QSpinBox()
        init_widget(spin_box, "spinBox")
        spin_box.setValue(50)

        date_timeedit = QDateTimeEdit()
        init_widget(date_timeedit, "dateTimeEdit")
        date_timeedit.setDateTime(QDateTime.currentDateTime())

        slider = QSlider()
        init_widget(slider, "slider")
        slider.setOrientation(Qt.Orientation.Horizontal)
        slider.setValue(40)

        scrollbar = QScrollBar()
        init_widget(scrollbar, "scrollBar")
        scrollbar.setOrientation(Qt.Orientation.Horizontal)
        scrollbar.setValue(60)

        dial = QDial()
        init_widget(dial, "dial")
        dial.setValue(30)
        dial.setNotchesVisible(True)

        layout = QGridLayout(result)
        layout.addWidget(lineedit, 0, 0, 1, 2)
        layout.addWidget(spin_box, 1, 0, 1, 2)
        layout.addWidget(date_timeedit, 2, 0, 1, 2)
        layout.addWidget(slider, 3, 0)
        layout.addWidget(scrollbar, 4, 0)
        layout.addWidget(dial, 3, 1, 2, 1)
        layout.setRowStretch(5, 1)
        return result

    def create_progress_bar(self):
        result = QProgressBar()
        init_widget(result, "progressBar")
        result.setRange(0, 10000)
        result.setValue(0)

        timer = QTimer(self)
        timer.timeout.connect(self.advance_progressbar)
        timer.start(1000)
        return result

    @Slot()
    def update_systeminfo(self):
        """Display system information"""
        system_info = SYSTEMINFO.format(sys.version,
                                        QLibraryInfo.build(),
                                        QSysInfo.prettyProductName(),
                                        QGuiApplication.platformName(),
                                        screen_info(self))
        self._systeminfo_textbrowser.setHtml(system_info)

    @Slot()
    def help_on_current_widget(self):
        """Display help on widget under mouse"""
        w = QApplication.widgetAt(QCursor.pos(self.screen()))
        while w:  # Skip over internal widgets
            name = w.objectName()
            if name and not name.startswith("qt_"):
                launch_help(w)
                break
            w = w.parentWidget()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(True)

from PySide6.QtCore import Qt
from testbinding import Enum1, TestObjectWithoutNamespace

import dis


class ListConnectionTest(unittest.TestCase):

    def testEnumVisibility(self):
        self.assertEqual(Enum1.Option1.value, 1)
        self.assertEqual(Enum1.Option2.value, 2)
        self.assertEqual(TestObjectWithoutNamespace.Enum2.Option3, 3)
        self.assertEqual(TestObjectWithoutNamespace.Enum2.Option4, 4)

    def testFlagComparisonOperators(self):  # PYSIDE-1696, compare to self
        f1 = Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignBottom
        f2 = Qt.AlignmentFlag.AlignHCenter | Qt.AlignmentFlag.AlignBottom
        self.assertTrue(f1 == f1)
        self.assertTrue(f1 <= f1)
        self.assertTrue(f1 >= f1)
        self.assertFalse(f1 != f1)
        self.assertFalse(f1 < f1)
        self.assertFalse(f1 > f1)

        self.assertTrue(f1 == f2)
        self.assertTrue(f1 <= f2)
        self.assertTrue(f1 >= f2)
        self.assertFalse(f1 != f2)
        self.assertFalse(f1 < f2)
        self.assertFalse(f1 > f2)

        self.assertTrue(Qt.AlignmentFlag.AlignHCenter < Qt.AlignmentFlag.AlignBottom)
        self.assertFalse(Qt.AlignmentFlag.AlignHCenter > Qt.AlignmentFlag.AlignBottom)
        self.assertFalse(Qt.AlignmentFlag.AlignBottom < Qt.AlignmentFlag.AlignHCenter)
        self.assertTrue(Qt.AlignmentFlag.AlignBottom > Qt.AlignmentFlag.AlignHCenter)


# PYSIDE-1735: We are testing that opcodes do what they are supposed to do.
#              This is needed in the PyEnum forgiveness mode (ENOPT_NO_ZERODEFAULT)
#              where we need to introspect the code if an Enum was called with no args,
#              enabling default construction like 'f = Qt.WindowFlags()'.
#              Adapt for each Python version by checking the defines in the generated header opcode_ids.h
# egrep '( RESUME | LOAD_GLOBAL | LOAD_ATTR | PUSH_NULL | CALL | STORE_FAST | RETURN_CONST )' opcode_ids.h
# See also sbkfeature_base.cpp

# flake8: noqa
class InvestigateOpcodesTest(unittest.TestCase):

    def probe_function1(self):
        x = Qt.Alignment

    def probe_function2(self):
        x = Qt.Alignment()

    @staticmethod
    def read_code(func, **kw):
        return list(instr[:3] for instr in dis.Bytecode(func, **kw))

    @staticmethod
    def get_sizes(func, **kw):
        ops = list((instr.opname, instr.offset) for instr in dis.Bytecode(func, **kw))
        res = []
        for idx in range(1, len(ops)):
            res.append((ops[idx - 1][0], ops[idx][1] - ops[idx - 1][1]))
        return sorted(res, key=lambda x: (x[1], x[0]))

    _sin = sys.implementation.name

    def testByteCode(self):
        import dis
        # opname, opcode, arg
        result_1 = [('LOAD_GLOBAL', 116, 0),
                    ('LOAD_ATTR',   106, 1),
                    ('STORE_FAST',  125, 1),
                    ('LOAD_CONST',  100, 0),
                    ('RETURN_VALUE', 83, None)]

        result_2 = [('LOAD_GLOBAL', 116, 0),
                    ('LOAD_METHOD', 160, 1),
                    ('CALL_METHOD', 161, 0),
                    ('STORE_FAST',  125, 1),
                    ('LOAD_CONST',  100, 0),
                    ('RETURN_VALUE', 83, None)]

        if sys.version_info[:2] == (3, 11):
            # Note: Python 3.11 is a bit more complex because it can optimize itself.
            # Opcodes are a bit different, and a hidden second code object is used.
            # We investigate this a bit, because we want to be warned when things change.
            QUICKENING_WARMUP_DELAY = 8

            result_1 = [('RESUME',      151, 0),
                        ('LOAD_GLOBAL', 116, 0),
                        ('LOAD_ATTR',   106, 1),
                        ('STORE_FAST',  125, 1),
                        ('LOAD_CONST',  100, 0),
                        ('RETURN_VALUE', 83, None)]

            result_2 = [('RESUME',      151, 0),
                        ('LOAD_GLOBAL', 116, 1),
                        ('LOAD_ATTR',   106, 1),
                        ('PRECALL',     166, 0),
                        ('CALL',        171, 0),
                        ('STORE_FAST',  125, 1),
                        ('LOAD_CONST',  100, 0),
                        ('RETURN_VALUE', 83, None)]

            sizes_2 = [('LOAD_CONST',   2),
                       ('RESUME',       2),
                       ('STORE_FAST',   2),
                       ('PRECALL',      4),
                       ('CALL',        10),
                       ('LOAD_ATTR',   10),
                       ('LOAD_GLOBAL', 12)]

            self.assertEqual(self.read_code(self.probe_function2, adaptive=True), result_2)
            self.assertEqual(self.get_sizes(self.probe_function2, adaptive=True), sizes_2)

            @staticmethod
            def code_quicken(f, times):
                # running the code triggers acceleration after some runs.
                for _ in range(times):
                    f()

            code_quicken(self.probe_function2, QUICKENING_WARMUP_DELAY - 1)
            self.assertEqual(self.read_code(self.probe_function2, adaptive=True), result_2)
            self.assertEqual(self.get_sizes(self.probe_function2, adaptive=True), sizes_2)

            result_3 = [('RESUME_QUICK',       150, 0),
                        ('LOAD_GLOBAL_MODULE',  55, 1),
                        ('LOAD_ATTR_ADAPTIVE',  39, 1),
                        ('PRECALL_ADAPTIVE',    64, 0),
                        ('CALL_ADAPTIVE',       22, 0),
                        ('STORE_FAST',         125, 1),
                        ('LOAD_CONST',         100, 0),
                        ('RETURN_VALUE',        83, None)]

            sizes_3 = [('LOAD_CONST',          2),
                       ('RESUME_QUICK',        2),
                       ('STORE_FAST',          2),
                       ('PRECALL_ADAPTIVE',    4),
                       ('CALL_ADAPTIVE',      10),
                       ('LOAD_ATTR_ADAPTIVE', 10),
                       ('LOAD_GLOBAL_MODULE', 12)]

            code_quicken(self.probe_function2, 1)
            self.assertEqual(self.read_code(self.probe_function2, adaptive=True), result_3)
            self.assertEqual(self.get_sizes(self.probe_function2, adaptive=True), sizes_3)

        if sys.version_info[:2] == (3, 12):

            result_1 = [('RESUME',       151, 0),
                        ('LOAD_GLOBAL',  116, 0),
                        ('LOAD_ATTR',    106, 2),
                        ('STORE_FAST',   125, 1),
                        ('RETURN_CONST', 121, 0)]

            result_2 = [('RESUME',       151, 0),
                        ('LOAD_GLOBAL',  116, 1),
                        ('LOAD_ATTR',    106, 2),
                        ('CALL',         171, 0),
                        ('STORE_FAST',   125, 1),
                        ('RETURN_CONST', 121, 0)]

        if sys.version_info[:2] == (3, 13):

            result_1 = [('RESUME',       149, 0),
                        ('LOAD_GLOBAL',   91, 0),
                        ('LOAD_ATTR',     82, 2),
                        ('STORE_FAST',   110, 1),
                        ('RETURN_CONST', 103, 0)]

            result_2 = [('RESUME',       149, 0),
                        ('LOAD_GLOBAL',   91, 0),
                        ('LOAD_ATTR',     82, 2),
                        ('PUSH_NULL',     34, None),
                        ('CALL',          53, 0),
                        ('STORE_FAST',   110, 1),
                        ('RETURN_CONST', 103, 0)]

        if sys.version_info[:2] >= (3, 14):

            result_1 = [('RESUME',       128, 0),
                        ('LOAD_GLOBAL',   92, 0),
                        ('LOAD_ATTR',     80, 2),
                        ('STORE_FAST',   112, 1),
                        ('LOAD_CONST',    82, 0),
                        ('RETURN_VALUE',  35, None)
                        ]

            result_2 = [('RESUME',       128, 0),
                        ('LOAD_GLOBAL',   92, 0),
                        ('LOAD_ATTR',     80, 2),
                        ('PUSH_NULL',     33, None),
                        ('CALL',          52, 0),
                        ('STORE_FAST',   112, 1),
                        ('LOAD_CONST',    82, 0),
                        ('RETURN_VALUE',  35, None)]

        self.assertEqual(self.read_code(self.probe_function1), result_1)
        self.assertEqual(self.read_code(self.probe_function2), result_2)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2024 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QObject
from PySide6.QtQml import QmlAnonymous


QML_IMPORT_NAME = "ColorPalette"
QML_IMPORT_MAJOR_VERSION = 1


@QmlAnonymous
class AbstractResource(QObject):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_manager = None  # QRestAccessManager
        self.m_api = None  # QNetworkRequestFactory

    def setAccessManager(self, manager):
        self.m_manager = manager

    def setServiceApi(self, serviceApi):
        self.m_api = serviceApi



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test weakref support'''

import gc
import os
import sys
import unittest
import weakref

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ObjectType, PrivateDtor


class WeakrefBasicTest(unittest.TestCase):
    '''Simple test case of using a weakref'''

    def setUp(self):
        self.called = False

    def cb(self, *args):
        self.called = True

    def testBasic(self):
        '''ObjectType weakref'''
        obj = ObjectType()
        ref = weakref.ref(obj, self.cb)  # noqa: F841
        del obj
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertTrue(self.called)

    def testPrivateDtor(self):
        '''PrivateDtor weakref'''
        obj = PrivateDtor.instance()
        ref = weakref.ref(obj, self.cb)  # noqa: F841
        del obj
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertTrue(self.called)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

from functools import partial
import os
import sys
import unittest

TEST_DIR = os.path.dirname(os.path.abspath(__file__))

sys.path.append(os.path.dirname(TEST_DIR))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtCore import QCoreApplication, QSize, QUrl  # noqa: E402
from PySide6.QtWidgets import QApplication, QVBoxLayout, QWidget  # noqa: E402
from PySide6.QtWebEngineCore import QWebEnginePage  # noqa: E402
from PySide6.QtWebEngineWidgets import QWebEngineView  # noqa: E402


class MainTest(unittest.TestCase):

    def test_WebEngineView_findText_exists(self):
        app = QApplication.instance() or QApplication()
        top_level = QWidget()
        layout = QVBoxLayout(top_level)
        self._view = QWebEngineView()
        self._view.loadFinished.connect(self.loaded)
        self._view.load(QUrl.fromLocalFile(os.path.join(TEST_DIR, "fox.html")))
        self._view.setMinimumSize(QSize(400, 300))
        self._callback_count = 0
        layout.addWidget(self._view)
        top_level.show()
        app.exec()

    def found_callback(self, found):
        self.assertTrue(found)
        self._callback_count += 1
        if self._callback_count == 2:
            QCoreApplication.quit()

    def javascript_callback(self, result):
        self.assertEqual(result, "Title")
        self._callback_count += 1
        if self._callback_count == 2:
            QCoreApplication.quit()

    def loaded(self, ok):
        self.assertTrue(ok)
        if not ok:
            QCoreApplication.quit()
        self._view.page().runJavaScript("document.title", 1,
                                        partial(self.javascript_callback))
        self._view.findText("fox", QWebEnginePage.FindFlag(0),
                            partial(self.found_callback))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

"""
testing/__init__.py

- install an alternative, flushing print
- define command.main as entry point
"""

import builtins
import sys

from . import command

main = command.main

# modify print so that it always flushes
builtins.orig_print = builtins.print


def print_flushed(*args, **kw):
    orig_print(*args, **kw)
    sys.stdout.flush()


builtins.print = print_flushed

print = print_flushed

# We also could use "python -u" to get unbuffered output.
# This method is better since it needs no change of the interface.

# eof



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import qAbs


class QAbsTest(unittest.TestCase):
    '''Tests for qAbs'''

    def testBasic(self):
        def check(x):
            self.assertEqual(qAbs(x), abs(x))

        check(0)
        check(-10)
        check(10)
        check(10.5)
        check(-10.5)


if __name__ == '__main__':
    unittest.main()



# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.10.0
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x04\xe8\
/\
/ Copyright (C) \
2021 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port QtQuick.Con\
trols\x0a\x0aColumn {\x0a\
    id: root\x0a   \
 spacing: 8\x0a\x0a   \
 Label {\x0a       \
 text: \x22Total nu\
mber of screens:\
 \x22 + screenInfo.\
count\x0a        fo\
nt.bold: true\x0a  \
  }\x0a\x0a    Flow {\x0a\
        spacing:\
 12\x0a        widt\
h: parent.width\x0a\
\x0a        Repeate\
r {\x0a            \
id: screenInfo\x0a \
           model\
: (Qt.applicatio\
n as Application\
).screens\x0a      \
      Label {\x0a  \
              re\
quired property \
string name\x0a    \
            requ\
ired property in\
t virtualX\x0a     \
           requi\
red property int\
 virtualY\x0a      \
          requir\
ed property var \
modelData // avo\
id shadowing Lab\
el.width and hei\
ght\x0a\x0a           \
     lineHeight:\
 1.5\x0a           \
     text: name \
+ \x22\x5cn\x22 + virtual\
X + \x22, \x22 + virtu\
alY + \x22 \x22 + mode\
lData.width + \x22x\
\x22 + modelData.he\
ight\x0a           \
 }\x0a        }\x0a   \
 }\x0a\x0a    Componen\
t.onCompleted: {\
\x0a        var scr\
eens = (Qt.appli\
cation as Applic\
ation).screens;\x0a\
        for (var\
 i = 0; i < scre\
ens.length; ++i)\
\x0a            con\
sole.log(\x22screen\
 \x22 + screens[i].\
name + \x22 has geo\
metry \x22 +\x0a      \
                \
  screens[i].vir\
tualX + \x22, \x22 + s\
creens[i].virtua\
lY + \x22 \x22 +\x0a     \
                \
   screens[i].wi\
dth + \x22x\x22 + scre\
ens[i].height)\x0a \
   }\x0a}\x0a\
\x00\x00\x05\x01\
(\
\xb5/\xfd`\xff\x16\xbd'\x00\x06p\x912`\x8d:\
0\x0c\xc3\xc0\x161\x1c\x00\x98G\xc1\xb5\xb6\x93$\xda\
IN\x11E,I\x22\xd5#\xdck\xe3\x5c\x9d/Y\
\xc2BlU\xb2\xcdF\x1e\x12Xb\x833\xad\x1c\x83\
\x00~\x00\x85\x00\x15\xc9\xe1\xec\x8b\xb7\xd4\x8d&Lf\
+\xd1\xec\xe70\xcf\xcb\xcc\x85\x18\xaetL\xb7\xf4\x94\
1\xa7\xd8\xbb\xd3\xad`\xef\xeeN \xfb\x8f7@\x13\
\xdf\x99\xa6*\x9d\xfet\x8b\xe2x\x9du\xbe\xb3O\x1d\
\xbbz\x9a\xb8fNZ\x96\x8d\xa5\xc1\xc3\xb0\x15\x13+\
]?\xca\x97\xe5;\x96\x10WlE\xb9C\xfd\x93!\
\x84\x85\xeb\xe4\xd7\x11\x16\x8c\x06\x0a\xcfD\xc1\x8f\xd8\xd2\
t\xd8\xe5\xd6~g9~6\xa3\xc1h\xb6bQ\xf6\
\xa3\xa8i\x09sk/\x97\x1c\x8f\xa3\x059c\x01.\
\x17\x11\x80M\x80\xcc\xf1ZZ\xd3\xa3\xa7m\xb5\x8da\
s\xda*,KP\xd7\x1eQ_\xf9I\x1e\xab\xffg\
\x0c\x9b'\x82M\x1fD\xb6\x91\x16N\x7f\x5c\x97\xe3\xbf\
\x07\xff\x82\xacd\x90\x8b\xa8k\xd9\xcd^\xf6BJ~\
\xb5\xaa\xb3E\xd4]?\x99J\xd9!\xaaR\x8fo\xba\
ye\xd2\xad\xd6\xb6b\xf9\xfd$\x8a\xffY:l+\
\xea\x1c\x8e\x9a\x15Xj`/\xbe_K\xecjJ\xfa\
\xdb*>l\xd2\x06_\xf9\xd30\x98\x02\x83\x8c&\xa0\
\x15h0\x9a\x8d\x08\xc8\xd6\x05\x10\x9aY\x19\x96\xa2\xe9\
\xfeqE\xa5\xba\xb8p\xbam\x03[\x99M\xd9\x81m\
[{I>9f\x16!l\x8e\xab\xa1\x0b\xa1\x91\xda\
N\x9a=\xd3\x8a\xab{\x1b\xdf7\xa9VF\x14\xd1l\
\xfa\x1e\xff\xec\xf4\xfe7\xb3\x90\x87\x83\xfc\xd1W\xcf\x96\
\xb0h\xaf\xdf]\xfe\xd9B\xf3\xcf\x5c\x9b\xcc\xf4yV\
\xca\xcca\x13\x12\x12\xb6\xf4-\x9b3\xe9F\x1c\xdfN\
\x87\xb3\x19\xa8\x0b\x0fg\xd0\xf0pP\x93H\xe6|\xb5\
'V'\xb1Z;\xfeV(\x5c]\xbe\x91\x92\x10\xcb\
\xad\xa8\x7fe\xda(\xf4S;VA\xba\x11\xe5\x9f\x10\
P:\x07\xce\x85Z\x99Rp\xcd\x9ce\xe9k\xc9I\
\x86\x90)\x8f<b\xfd\x91\xee\x9ffBrR)p\
F\x1f\xe40\xa74\xb4\xc9\x14\x1cP\x98\x9d\x084{\
\xd9\x0a\xe5r8\x1d\xa5\xe2;\x89\x1dX\x03\x0d\x85\xa1\
8\x90\x88&`\xd3)K8\x1d\x07s\x1f\xcf\x84\xa0\
\xfc\x93\xdf\xf4\xb39\x0e!\xd4w\xb6\xaa\x22'C\xf1\
\x03\x81 \xa8\x11\xa6\x19\x1a\x11\x19IR\x94\x0c\x07A\
\x08\x820\x14uV\x9a\x07Rx$IC\x18\xc3 \
\x14\x8d\x10)`\x06\x11D\x08\x99HH\x82\x0bd\x92\
4\xd5\x01\xfd\x86\xe3\x00;U\xd9[%\xb0+\xa0H\
Y\x22\xa8\xc45\x02LtZ\xa2\x1f\xf5yiy\xf2\
1\xe7\xffg\xce\x8eR\xb6\x88\xc1\xb6\x88m\xaf\xd4)\
\xd7\xa7\x00\xb5\x15Y,,MT\xd3\x84\xcc\xb2\xd8\xca\
\xa0\xd5&S\x9b&\x97\x05\xed\x06\x92\xfc]\xe6-i\
q\x1ckF\x85\xbc(\x08\xa3\x9c\xbd\x00\xb3~1Z\
\xf8\xd0\xa6d\xb1\x8a\x8f\xd7{\x9b\x97/=\xf0F\xf8\
\xb4L\xefw\xce\xcd\xd0\xdb\xe9\xc9\xc7\xa8\x08\xdd\x97\xb9\
\xfc\x035B\xc9\xfa\xcdKA\x83\xd7)\x8e\xdf\xb0\xd4\
\x04F\xe2\xe7\xca\xcf<v\xa5D\xa0\xbd\xc9x\x87E\
R\xb3@\x8cTv\x10\x04+\xc8\xd8\xc35\x07r\x18\
VCyf\xa6\x08q\x03\x10\x07<\xefF\xbd\xff\xf9\
\xff\x84\x9d\xfb\xa7\x88\xff\x0c\x0d\xed\xed\xb4\xd8\xa2\xe22\
\x10\xac\xd3PA\xe2\xf8\x17\x13;4\x9e\xfd1\x92\x11\
\xb5\xf6\x228t\xd6\x8c&\x87\xaf\xd0\xc3\xc3\xba\xa6\xa1\
\xb8\xb9N\xb9/\xf7\x05~\x22\xa5d,\xc1b\xd1#\
%U\x05\xfe\xc3\x14T\xbc0\xc9\xe7\x80PZ\xea_\
\xdd\x9eO@1\xf8\x10.\xf7\x8c\x1fe\xc0\xacHQ\
<\xb6\x026\xee\xcb\x81] 4\x0e%\xbe,\x0f\x87\
Jk\x91XN9\xb1\x14$\xa0\xad\xd8I\xc1Uy\
S\xb4\xae\xd4F\xf7X3\x92\x9e!E\xca$\xe7x\
\xaazGx\x0eG(\x8e\x0cmij\x0aL\xd3\xfd\
\xc0\x0a\xe4\xf3BKz\xa02G\xf6\x10\xca\x04D\x1b\
\xc9\x86\xe0\xa7^\xb5Dz@\xcd\x9c\xfa\xd3-\x97\x10\
a5\xb1\xba\x97D\x86\xb1\x0f\xea\x0c\x95\xf4tU\x1f\
\x93\xf7\xacR\x9c\xb6\x00\x83\xf4\xefK\xa5\xf8]\x97\x80\
'\xa2\x04\x17\xeeu\xf43\x1f\x81\xe0C\xee\xf4\xf7\x85\
7\xda17\x00\x1a\x14]\xdb\xb6\xad\x1c\x0d8\x12q\
\x9c:\x0c1\x22\xf3\xd1\x09Vz\x916\x01h\xd9\x88\
\x8b\xbf\xb5\xa3\xb84\xe2\xcd\xe7b\xf1\xac\xad\x9cR\x96\
\xe3\x11\x97\x92\x93\xa3\xc1\x87\xf2:\xf2y\x0b\xb0\xca\xbd\
\xdd \xc7t?D\xaf\x16-\x05a\xd2\xbd9 \xb3\
\x9fRI\xaf\xbc\x99\x1cC\x0eK?\x9e\xdc\x86\xc3$\
\xfe)\xab-\x01v\xbb\x1e\x8e\xdd\xfbm\x12\xc8\x1b\xf5\
\xce\x1fym\xd8\x10\x0e\xe5\xd5w\xd1\x05m\x14`\xd5\
\x0bAo)\xb0c,\x81'H\x81.xsAP\
=&\xe5\xa0Z\xe0\x828\xa6\xdc\xde\xa0\xeb\x8e\xb1F\
\xcb\xc0D6_,\x04\xa8y\xe0\xdf\x95\x9f\xfb\xe1\x97\
\xa3I\xa6\x1f#k)\xf4\xaf\x03m\xd7\xc8AP\x12\
]\x154_\xbd\x1cU\x0dcv\xb8\xbf\x1a\xc0\xa8\x02\
\
\x00\x00\x03\xd5\
/\
/ Copyright (C) \
2021 The Qt Comp\
any Ltd.\x0a// SPDX\
-License-Identif\
ier: LicenseRef-\
Qt-Commercial OR\
 BSD-3-Clause\x0a\x0ai\
mport QtQuick\x0aim\
port shared\x0a\x0a//!\
 [splash-propert\
ies]\x0aWindow {\x0a  \
  id: splash\x0a   \
 color: \x22transpa\
rent\x22\x0a    title:\
 \x22Splash Window\x22\
\x0a    modality: Q\
t.ApplicationMod\
al\x0a    flags: Qt\
.SplashScreen\x0a  \
  property int t\
imeoutInterval: \
2000\x0a    signal \
timeout\x0a//! [spl\
ash-properties]\x0a\
//! [screen-prop\
erties]\x0a    x: (\
Screen.width - s\
plashImage.width\
) / 2\x0a    y: (Sc\
reen.height - sp\
lashImage.height\
) / 2\x0a//! [scree\
n-properties]\x0a  \
  width: splashI\
mage.width\x0a    h\
eight: splashIma\
ge.height\x0a\x0a    I\
mage {\x0a        i\
d: splashImage\x0a \
       source: I\
mages.qtLogo\x0a   \
     TapHandler \
{\x0a            on\
Tapped: splash.e\
xit()\x0a        }\x0a\
    }\x0a\x0a    funct\
ion exit() {\x0a   \
     splash.visi\
ble = false\x0a    \
    splash.timeo\
ut()\x0a    }\x0a\x0a    \
//! [timer]\x0a    \
Timer {\x0a        \
interval: splash\
.timeoutInterval\
; running: splas\
h.visible; repea\
t: false\x0a       \
 onTriggered: sp\
lash.exit()\x0a    \
}\x0a    //! [timer\
]\x0a}\x0a\
\x00\x00\x03\x03\
(\
\xb5/\xfd`d\x09\xcd\x17\x00fbg\x1f\x00\xdb6\
\x8e\x19\x94\xa4Zd_\xd9sQ1d\xb4\x22\x92\x81\
P\xe2a+\xcaH\xcc\xa5\xff\xed?\x94`\x00\x5c\x00\
^\x00\xddtf\xe6\x0c\xc6^?&\xcb@j\xefW\
;\xd9V\xd2\xc3\xcew\xf8\xaf\xfd<\xcf\xb2\x06\xc3H\
'\x06\x85\x81\xf2\x91\xb0g\xc34\xba\xbc\xed\x19\xbec\
@\xb5\x87\xbb\xdb\x8e\xf2\xaf=1\x86\x06kK\xcf\x18\
D\x14\xce'\xdb\xe6g\xf8U\xe3\xe7\xd9\x09\x7f\x87[\
\x05bD\xe2\x90`\xc1$\xc0\xafJ\xf9}}v\xc2\
\x1c\x17\xb6\xaej\x98\xc5\xd8g\xffmV\xc5\xd0\xfe\xa2\
\x04g\x8d\x99\xf7x\xff\xda\x87\xf6i\x5co\xf8}9\
\xeb6\xaf\x1b=\xbf\x9c\x0d\x8c\x92U\xf2\xbd\x05\x0al\
 U\xfd\xfe\xc7\x82\xe2\xd7\xf6k\xb9Z\x7fYCT\
\xd6@0\x81/\xef\xd2\xe96\xfdZ^\x7f\xfb\xb3\xf7\
\xe3\xaeU\x87\xc3\x82;\xd5Oo\xbd\xb5\x97\x03\xb5=\
\x9f\x9a\x93\x17V\xa6\xd1o\x83\xb4\xa7\x8dq\x07\xe1\x18\
k\xb7o\xf82\xa2\xc2\x1a\xbd\xec6\xc7q\x8c\x11\x0f\
D\x87\xf7\xb9\x9b\xd4\x9e\xb6\xaf\x16\xa5uZIc\x02\
\xb7\x17\xeb\xc6\x04\xd6\xb4\x12\x7f4\xee\xde\xd5O\xfb\x14\
3\xa6\xda}%Hb\x00\xc7\xbd:\x97\x83\xbbTu\
\x19\xbf\x04w`\xf5Bj\xa7\xc7j+Y\xec\x80\x98\
}\xaf\xf54M\xd5\xf8\xb2/{\xde\x01\xb1\xe1\xed\xad\
\xbd#.\x88\xaa;h/\xa1\xc8g\x7f\x05\xc6\xf0\xaf\
\xb3\xb8\xe9)R\x8f\xac\xe8\xb1\xfe\xf1\xb6tVO\x0e\
\xd3(=e\xa9@%\xab\x0b\x16\x1b#\xc1\x03\x0e\x04\
\xd8\xf38\xf0\xeddA\x220\xdb\xe5\x15\xfb\x1b\xe5\xf5\
\x93i\xbb\xfcs\x81\xd5:\x18\x80\x93\xa0\xa1\xa532\
C3\x92$)\xa5\x03 \x84\x10\xc4\xdcq\x07\xe9\x80\
\xaa\xec\xf5}\xa7\x0a~\xe6v j\x95\xc9\x9a\x9f\x93\
\xb1\x1fxx\x8dL\x93\x8d$fa.\xaf\x96kF\
\x7fRr.\x00r\x00\xedF2\x85\xee\x9e\x1a\x02H\
\xfc\x150\x0f\x05'x\xbb\x5c\x19\xc8p\xef\xa2\xad\xb2\
'W\xec]a^\x5cb\x0f\x22\xcb\xd4\x0d+\x15\x8c\
\xe4lK\x86\x11\xea$T\x9a'\xf2\xae\xe7BL\xee\
\xc5k\xac!\xbf8\x8d5D\xb0\x0e\xed\xd7\x91\x83\xbb\
\x880\xa8mr\xbb\xc2\xd9\x0cO+\xd34\x80\xc9\xbc\
o\x14Y\x0b\x883O\x10\xfc\x88]p>\x8cr\x8d\
T\xf0k\xf8\x0bj\xef?\xd0r\xb57\x1d\x07(\xe7\
\xee{h\x13\x8b\xc1\xb6\x90\x92\xf9\x8a\xe5\x95)\xd9\x22\
2\xc6\xc9e\xe7i\xb0R\x1f\x18W\x95lPV#\
}Mgl\xb3\xb8\xa7\xd8EF\x22\xaa\xe40P8\
\xf1\xaf\x0f\x82\x0dt\xe6\xa4\x01G@\x07\xa5;^\x0c\
v\x14\xe4\x90k*\x5c\xfe\x9b\x14o\xd6@9\xd6\xa2\
\x7f\xb0\x85\x1f\xe7yV2\xe4+\xf4\xcc1S<\x98\
\x01\x9e<Q]\xa5R\xc8\xc4\x84\x06\x22!\x8f\x0f$\
\xcc\xec\x19pP\xe8\xed\xc2\x83\xa3\xc1H.\xd7\x80\x85\
\x91K\xe8\xdc\x9cW\xf2V\x8a\x15\x02\xbeH,\xe0<\
\x16\x8c\xbbSP\xfb_\xd8)}\x00u\xcc(5\xe0\
U\x01\
"

qt_resource_name = b"\
\x00\x06\
\x07\xe0Kg\
\x00w\
\x00i\x00n\x00d\x00o\x00w\
\x00\x0e\
\x0ei\x17\x5c\
\x00A\
\x00l\x00l\x00S\x00c\x00r\x00e\x00e\x00n\x00s\x00.\x00q\x00m\x00l\
\x00\x0a\
\x0bi\x98\xbc\
\x00w\
\x00i\x00n\x00d\x00o\x00w\x00.\x00q\x00m\x00l\
\x00\x0a\
\x08\x90\x1a|\
\x00S\
\x00p\x00l\x00a\x00s\x00h\x00.\x00q\x00m\x00l\
\x00\x11\
\x02YG\x1c\
\x00C\
\x00u\x00r\x00r\x00e\x00n\x00t\x00S\x00c\x00r\x00e\x00e\x00n\x00.\x00q\x00m\x00l\
\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x04\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00h\x00\x04\x00\x00\x00\x01\x00\x00\x0d\xca\
\x00\x00\x01\x975l\xc7\xec\
\x00\x00\x00N\x00\x00\x00\x00\x00\x01\x00\x00\x09\xf1\
\x00\x00\x01\x998\xf9\x98B\
\x00\x00\x004\x00\x04\x00\x00\x00\x01\x00\x00\x04\xec\
\x00\x00\x01\x998\xf9\x98B\
\x00\x00\x00\x12\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01\x975l\xc7\xec\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QRandomGenerator, QRandomGenerator64


class QRandomGeneratorTest(unittest.TestCase):
    '''Test case for QRandomGenerator'''

    def testGenerator(self):
        self.assertTrue(QRandomGenerator.system())
        self.assertTrue(QRandomGenerator.global_())
        generator = QRandomGenerator()
        r = generator.bounded(10, 20)
        self.assertTrue(r >= 10)
        self.assertTrue(r <= 20)

    def testGenerator64(self):
        generator = QRandomGenerator64()
        r = generator.generate()  # noqa: F841


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test case for QObject.timerEvent overloading'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject

from helper.usesqapplication import UsesQApplication


class Dummy(QObject):

    def __init__(self, app):
        super().__init__()
        self.times_called = 0
        self.app = app

    def timerEvent(self, event):
        QObject.timerEvent(self, event)
        event.accept()
        self.times_called += 1

        if self.times_called == 5:
            self.app.exit(0)


class QObjectTimerEvent(UsesQApplication):

    def setUp(self):
        # Acquire resources
        super(QObjectTimerEvent, self).setUp()

    def tearDown(self):
        # Release resources
        super(QObjectTimerEvent, self).tearDown()

    def testTimerEvent(self):
        # QObject.timerEvent overloading
        obj = Dummy(self.app)
        timer_id = obj.startTimer(200)
        self.app.exec()
        obj.killTimer(timer_id)
        self.assertEqual(obj.times_called, 5)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for protected methods.'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import wrapperCount
from sample import ProtectedNonPolymorphic, ProtectedVirtualDestructor
from sample import (ProtectedPolymorphic, ProtectedPolymorphicDaughter,
                    ProtectedPolymorphicGrandDaughter)
from sample import createProtectedProperty, ProtectedProperty, ProtectedEnumClass
from sample import PrivateDtor
from sample import Event, ObjectType, Point


class ExtendedProtectedPolymorphic(ProtectedPolymorphic):
    def __init__(self, name):
        ProtectedPolymorphic.__init__(self, name)
        self.protectedName_called = False

    def protectedName(self):
        self.protectedName_called = True
        self._name = 'Extended' + ProtectedPolymorphic.protectedName(self)
        return self._name


class ExtendedProtectedPolymorphicDaughter(ProtectedPolymorphicDaughter):
    def __init__(self, name):
        self.protectedName_called = False
        ProtectedPolymorphicDaughter.__init__(self, name)

    def protectedName(self):
        self.protectedName_called = True
        self._name = 'ExtendedDaughter' + ProtectedPolymorphicDaughter.protectedName(self)
        return self._name


class ExtendedProtectedPolymorphicGrandDaughter(ProtectedPolymorphicGrandDaughter):
    def __init__(self, name):
        self.protectedName_called = False
        ProtectedPolymorphicGrandDaughter.__init__(self, name)

    def protectedName(self):
        self.protectedName_called = True
        self._name = 'ExtendedGrandDaughter' + ProtectedPolymorphicGrandDaughter.protectedName(self)
        return self._name


class ExtendedProtectedVirtualDestructor(ProtectedVirtualDestructor):
    def __init__(self):
        ProtectedVirtualDestructor.__init__(self)


class ProtectedNonPolymorphicTest(unittest.TestCase):
    '''Test cases for protected method in a class without virtual methods.'''

    def tearDown(self):
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertEqual(wrapperCount(), 0)

    def testProtectedCall(self):
        '''Calls a non-virtual protected method.'''
        p = ProtectedNonPolymorphic('NonPoly')
        self.assertEqual(p.publicName(), p.protectedName())
        a0, a1 = 1, 2
        self.assertEqual(p.protectedSum(a0, a1), a0 + a1)

    def testProtectedCallWithInstanceCreatedOnCpp(self):
        '''Calls a non-virtual protected method on an instance created in C++.'''
        p = ProtectedNonPolymorphic.create()
        self.assertEqual(p.publicName(), p.protectedName())
        a0, a1 = 1, 2
        self.assertEqual(p.protectedSum(a0, a1), a0 + a1)

    def testModifiedProtectedCall(self):
        '''Calls a non-virtual protected method modified with code injection.'''
        p = ProtectedNonPolymorphic('NonPoly')
        self.assertEqual(p.dataTypeName(), 'integer')
        self.assertEqual(p.dataTypeName(1), 'integer')
        self.assertEqual(p.dataTypeName(Point(1, 2)), 'pointer')


class ProtectedPolymorphicTest(unittest.TestCase):
    '''Test cases for protected method in a class with virtual methods.'''

    def tearDown(self):
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertEqual(wrapperCount(), 0)

    def testProtectedCall(self):
        '''Calls a virtual protected method.'''
        p = ProtectedNonPolymorphic('Poly')
        self.assertEqual(p.publicName(), p.protectedName())
        a0, a1 = 1, 2
        self.assertEqual(p.protectedSum(a0, a1), a0 + a1)

    def testProtectedCallWithInstanceCreatedOnCpp(self):
        '''Calls a virtual protected method on an instance created in C++.'''
        p = ProtectedPolymorphic.create()
        self.assertEqual(p.publicName(), p.protectedName())
        self.assertEqual(p.callProtectedName(), p.protectedName())

    def testReimplementedProtectedCall(self):
        '''Calls a reimplemented virtual protected method.'''
        original_name = 'Poly'
        p = ExtendedProtectedPolymorphic(original_name)
        name = p.callProtectedName()
        self.assertTrue(p.protectedName_called)
        self.assertEqual(p.protectedName(), name)
        self.assertEqual(ProtectedPolymorphic.protectedName(p), original_name)


class ProtectedPolymorphicDaugherTest(unittest.TestCase):
    '''Test cases for protected method in a class inheriting for a class with virtual methods.'''

    def testProtectedCallWithInstanceCreatedOnCpp(self):
        '''Calls a virtual protected method from parent class on an instance created in C++.'''
        p = ProtectedPolymorphicDaughter.create()
        self.assertEqual(p.publicName(), p.protectedName())
        self.assertEqual(p.callProtectedName(), p.protectedName())

    def testReimplementedProtectedCall(self):
        '''Calls a reimplemented virtual protected method from parent class.'''
        original_name = 'Poly'
        p = ExtendedProtectedPolymorphicDaughter(original_name)
        name = p.callProtectedName()
        self.assertTrue(p.protectedName_called)
        self.assertEqual(p.protectedName(), name)
        self.assertEqual(ProtectedPolymorphicDaughter.protectedName(p), original_name)


class ProtectedPolymorphicGrandDaugherTest(unittest.TestCase):
    '''Test cases for protected method in a class inheriting for a class that inherits from
    another with protected virtual methods.'''

    def tearDown(self):
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertEqual(wrapperCount(), 0)

    def testProtectedCallWithInstanceCreatedOnCpp(self):
        '''Calls a virtual protected method from parent class on an instance created in C++.'''
        p = ProtectedPolymorphicGrandDaughter.create()
        self.assertEqual(p.publicName(), p.protectedName())
        self.assertEqual(p.callProtectedName(), p.protectedName())

    def testReimplementedProtectedCall(self):
        '''Calls a reimplemented virtual protected method from parent class.'''
        original_name = 'Poly'
        p = ExtendedProtectedPolymorphicGrandDaughter(original_name)
        name = p.callProtectedName()
        self.assertTrue(p.protectedName_called)
        self.assertEqual(p.protectedName(), name)
        self.assertEqual(ProtectedPolymorphicGrandDaughter.protectedName(p), original_name)


class ProtectedVirtualDtorTest(unittest.TestCase):
    '''Test cases for protected virtual destructor.'''

    def setUp(self):
        ProtectedVirtualDestructor.resetDtorCounter()

    def tearDown(self):
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertEqual(wrapperCount(), 0)

    def testVirtualProtectedDtor(self):
        '''Original protected virtual destructor is being called.'''
        dtor_called = ProtectedVirtualDestructor.dtorCalled()
        for i in range(1, 10):
            pvd = ProtectedVirtualDestructor()
            # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
            gc.collect()
            del pvd
            # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
            gc.collect()
            self.assertEqual(ProtectedVirtualDestructor.dtorCalled(), dtor_called + i)

    def testVirtualProtectedDtorOnCppCreatedObject(self):
        '''Original protected virtual destructor is being called for a C++ created object.'''
        dtor_called = ProtectedVirtualDestructor.dtorCalled()
        for i in range(1, 10):
            pvd = ProtectedVirtualDestructor.create()
            del pvd
            # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
            gc.collect()
            self.assertEqual(ProtectedVirtualDestructor.dtorCalled(), dtor_called + i)

    def testProtectedDtorOnDerivedClass(self):
        '''Original protected virtual destructor is being called for a derived class.'''
        dtor_called = ExtendedProtectedVirtualDestructor.dtorCalled()
        for i in range(1, 10):
            pvd = ExtendedProtectedVirtualDestructor()
            del pvd
            # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
            gc.collect()
            self.assertEqual(ExtendedProtectedVirtualDestructor.dtorCalled(), dtor_called + i)


class ExtendedProtectedEnumClass(ProtectedEnumClass):
    def __init__(self):
        ProtectedEnumClass.__init__(self)

    def protectedEnumMethod(self, value):
        if value == ProtectedEnumClass.ProtectedEnum.ProtectedItem0:
            return ProtectedEnumClass.ProtectedEnum.ProtectedItem1
        return ProtectedEnumClass.ProtectedEnum.ProtectedItem0

    def publicEnumMethod(self, value):
        if value == ProtectedEnumClass.PublicEnum.PublicItem0:
            return ProtectedEnumClass.PublicEnum.PublicItem1
        return ProtectedEnumClass.PublicEnum.PublicItem0


class ProtectedEnumTest(unittest.TestCase):
    '''Test cases for protected enum.'''

    def tearDown(self):
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertEqual(wrapperCount(), 0)

    def testProtectedMethodWithProtectedEnumArgument(self):
        '''Calls protected method with protected enum argument.'''
        obj = ProtectedEnumClass()

        self.assertEqual(type(ProtectedEnumClass.ProtectedEnum.ProtectedItem0),
                         ProtectedEnumClass.ProtectedEnum)

        self.assertEqual(obj.protectedEnumMethod(ProtectedEnumClass.ProtectedEnum.ProtectedItem0),
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem0)
        self.assertEqual(obj.protectedEnumMethod(ProtectedEnumClass.ProtectedEnum.ProtectedItem1),
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem1)
        self.assertEqual(obj.callProtectedEnumMethod(ProtectedEnumClass.ProtectedEnum.ProtectedItem0),  # noqa: E501
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem0)
        self.assertEqual(obj.callProtectedEnumMethod(ProtectedEnumClass.ProtectedEnum.ProtectedItem1),  # noqa: E501
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem1)

    def testProtectedMethodWithPublicEnumArgument(self):
        '''Calls protected method with public enum argument.'''
        obj = ProtectedEnumClass()

        self.assertEqual(obj.publicEnumMethod(ProtectedEnumClass.PublicEnum.PublicItem0),
                         ProtectedEnumClass.PublicEnum.PublicItem0)
        self.assertEqual(obj.publicEnumMethod(ProtectedEnumClass.PublicEnum.PublicItem1),
                         ProtectedEnumClass.PublicEnum.PublicItem1)

        self.assertEqual(obj.callPublicEnumMethod(ProtectedEnumClass.PublicEnum.PublicItem0),
                         ProtectedEnumClass.PublicEnum.PublicItem0)
        self.assertEqual(obj.callPublicEnumMethod(ProtectedEnumClass.PublicEnum.PublicItem1),
                         ProtectedEnumClass.PublicEnum.PublicItem1)

    def testOverriddenProtectedMethodWithProtectedEnumArgument(self):
        '''Calls overridden protected method with protected enum argument.'''
        obj = ExtendedProtectedEnumClass()

        self.assertEqual(obj.protectedEnumMethod(ProtectedEnumClass.ProtectedEnum.ProtectedItem0),
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem1)
        self.assertEqual(obj.protectedEnumMethod(ProtectedEnumClass.ProtectedEnum.ProtectedItem1),
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem0)

        self.assertEqual(ProtectedEnumClass.protectedEnumMethod(obj, ProtectedEnumClass.ProtectedEnum.ProtectedItem0),  # noqa: E501
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem0)
        self.assertEqual(ProtectedEnumClass.protectedEnumMethod(obj,
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem1), ProtectedEnumClass.ProtectedEnum.ProtectedItem1)  # noqa: E501

        self.assertEqual(obj.callProtectedEnumMethod(ProtectedEnumClass.ProtectedEnum.ProtectedItem0),  # noqa: E501
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem1)
        self.assertEqual(obj.callProtectedEnumMethod(ProtectedEnumClass.ProtectedEnum.ProtectedItem1),  # noqa: E501
                         ProtectedEnumClass.ProtectedEnum.ProtectedItem0)

    def testOverriddenProtectedMethodWithPublicEnumArgument(self):
        '''Calls overridden protected method with public enum argument.'''
        obj = ExtendedProtectedEnumClass()

        self.assertEqual(obj.publicEnumMethod(ProtectedEnumClass.PublicEnum.PublicItem0),
                         ProtectedEnumClass.PublicEnum.PublicItem1)
        self.assertEqual(obj.publicEnumMethod(ProtectedEnumClass.PublicEnum.PublicItem1),
                         ProtectedEnumClass.PublicEnum.PublicItem0)

        self.assertEqual(ProtectedEnumClass.publicEnumMethod(obj, ProtectedEnumClass.PublicEnum.PublicItem0),  # noqa: E501
                         ProtectedEnumClass.PublicEnum.PublicItem0)
        self.assertEqual(ProtectedEnumClass.publicEnumMethod(obj, ProtectedEnumClass.PublicEnum.PublicItem1),  # noqa: E501
                         ProtectedEnumClass.PublicEnum.PublicItem1)

        self.assertEqual(obj.callPublicEnumMethod(ProtectedEnumClass.PublicEnum.PublicItem0),
                         ProtectedEnumClass.PublicEnum.PublicItem1)
        self.assertEqual(obj.callPublicEnumMethod(ProtectedEnumClass.PublicEnum.PublicItem1),
                         ProtectedEnumClass.PublicEnum.PublicItem0)


class ProtectedPropertyTest(unittest.TestCase):
    '''Test cases for a class with a protected property (or field in C++).'''

    def setUp(self):
        self.obj = ProtectedProperty()

    def tearDown(self):
        del self.obj
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertEqual(wrapperCount(), 0)

    def testProtectedProperty(self):
        '''Writes and reads a protected integer property.'''
        self.obj.protectedProperty = 3
        self.assertEqual(self.obj.protectedProperty, 3)

    def testProtectedContainerProperty(self):
        '''Writes and reads a protected list of integers property.'''
        lst = [1, 2, 3, 4]
        self.obj.protectedContainerProperty = lst
        self.assertEqual(self.obj.protectedContainerProperty, lst)

    def testProtectedEnumProperty(self):
        '''Writes and reads a protected enum property.'''
        self.obj.protectedEnumProperty = Event.EventType.SOME_EVENT
        self.assertEqual(self.obj.protectedEnumProperty, Event.EventType.SOME_EVENT)

    def testProtectedValueTypeProperty(self):
        '''Writes and reads a protected value type property.'''
        point = Point(12, 34)
        self.obj.protectedValueTypeProperty = point
        self.assertEqual(self.obj.protectedValueTypeProperty, point)
        self.assertFalse(self.obj.protectedValueTypeProperty is point)
        pointProperty = self.obj.protectedValueTypeProperty
        self.assertTrue(self.obj.protectedValueTypeProperty is pointProperty)

    def testProtectedValueTypePropertyWrapperRegistration(self):
        '''Access colocated protected value type property.'''
        wrapper_count = wrapperCount()
        point = Point(12, 34)
        obj = createProtectedProperty()
        obj.protectedValueTypeProperty
        self.assertEqual(obj.protectedValueTypeProperty.copy(),
                         obj.protectedValueTypeProperty)
        obj.protectedValueTypeProperty = point
        self.assertEqual(obj.protectedValueTypeProperty, point)
        self.assertFalse(obj.protectedValueTypeProperty is point)
        pointProperty = obj.protectedValueTypeProperty
        self.assertTrue(obj.protectedValueTypeProperty is pointProperty)
        del obj, point, pointProperty
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertEqual(wrapperCount(), wrapper_count)

    def testProtectedValueTypePointerProperty(self):
        '''Writes and reads a protected value type pointer property.'''
        pt1 = Point(12, 34)
        pt2 = Point(12, 34)
        self.obj.protectedValueTypePointerProperty = pt1
        self.assertEqual(self.obj.protectedValueTypePointerProperty, pt1)
        self.assertEqual(self.obj.protectedValueTypePointerProperty, pt2)
        self.assertTrue(self.obj.protectedValueTypePointerProperty is pt1)
        self.assertFalse(self.obj.protectedValueTypePointerProperty is pt2)
        # PYSIDE-535: Need to assign None to break the cycle
        self.obj.protectedValueTypePointerProperty = None

    def testProtectedObjectTypeProperty(self):
        '''Writes and reads a protected object type property.'''
        obj = ObjectType()
        self.obj.protectedObjectTypeProperty = obj
        self.assertEqual(self.obj.protectedObjectTypeProperty, obj)
        # PYSIDE-535: Need to assign None to break the cycle
        self.obj.protectedObjectTypeProperty = None


class PrivateDtorProtectedMethodTest(unittest.TestCase):
    '''Test cases for classes with private destructors and protected methods.'''

    def tearDown(self):
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        self.assertEqual(wrapperCount(), 0)

    def testProtectedMethod(self):
        '''Calls protected method of a class with a private destructor.'''
        obj = PrivateDtor.instance()

        self.assertEqual(type(obj), PrivateDtor)
        self.assertEqual(obj.instanceCalls(), 1)
        self.assertEqual(obj.instanceCalls(), obj.protectedInstanceCalls())
        obj = PrivateDtor.instance()
        self.assertEqual(obj.instanceCalls(), 2)
        self.assertEqual(obj.instanceCalls(), obj.protectedInstanceCalls())


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, QCoreApplication, ClassInfo


class TestClassInfo(unittest.TestCase):
    def test_metadata(self):
        @ClassInfo(author='pyside', url='http://www.pyside.org')
        class MyObject(QObject):
            pass

        o = MyObject()
        mo = o.metaObject()
        self.assertEqual(mo.classInfoCount(), 2)

        ci = mo.classInfo(0)  # author
        self.assertEqual(ci.name(), 'author')
        self.assertEqual(ci.value(), 'pyside')

        ci = mo.classInfo(1)  # url
        self.assertEqual(ci.name(), 'url')
        self.assertEqual(ci.value(), 'http://www.pyside.org')

    def test_dictionary(self):
        @ClassInfo({'author': 'pyside', 'author company': 'The Qt Company'})
        class MyObject(QObject):
            pass

        o = MyObject()
        mo = o.metaObject()
        self.assertEqual(mo.classInfoCount(), 2)

        ci = mo.classInfo(0)  # author
        self.assertEqual(ci.name(), 'author')
        self.assertEqual(ci.value(), 'pyside')

        ci = mo.classInfo(1)  # url
        self.assertEqual(ci.name(), 'author company')
        self.assertEqual(ci.value(), 'The Qt Company')

    def test_verify_metadata_types(self):
        valid_dict = {'123': '456'}

        invalid_dict_1 = {'123': 456}
        invalid_dict_2 = {123: 456}
        invalid_dict_3 = {123: '456'}

        ClassInfo(**valid_dict)

        self.assertRaises(TypeError, ClassInfo, **invalid_dict_1)

        # assertRaises only allows for string keywords, so a `try` must be used here.
        try:
            ClassInfo(**invalid_dict_2)
            self.fail('ClassInfo() accepted invalid_dict_2!')
        except TypeError:
            pass

        try:
            ClassInfo(**invalid_dict_3)
            self.fail('ClassInfo() accepted invalid_dict_3!')
        except TypeError:
            pass

    def test_can_not_use_instance_twice(self):
        decorator = ClassInfo(author='pyside', url='http://www.pyside.org')

        @decorator
        class MyObject1(QObject):
            pass

        class MyObject2(QObject):
            pass

        self.assertRaises(TypeError, decorator, MyObject2)

    def test_can_only_be_used_on_qobjects(self):
        def make_info():
            return ClassInfo(author='pyside')

        def test_function():
            pass
        self.assertRaises(TypeError, make_info(), test_function)

        class NotAQObject:
            pass
        self.assertRaises(TypeError, make_info(), NotAQObject)

        class QObjectSubclass(QObject):
            pass
        make_info()(QObjectSubclass)

        class SubclassOfNativeQObjectSubclass(QCoreApplication):
            pass
        make_info()(SubclassOfNativeQObjectSubclass)

        class SubclassOfPythonQObjectSubclass(QObjectSubclass):
            pass
        make_info()(SubclassOfPythonQObjectSubclass)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for libsample bindings module'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

import sample


class ModuleTest(unittest.TestCase):
    '''Test case for module and global functions'''

    def testAddedFunctionAtModuleLevel(self):
        '''Calls function added to module from type system description.'''
        str1 = 'Foo'
        self.assertEqual(sample.multiplyString(str1, 3), str1 * 3)
        self.assertEqual(sample.multiplyString(str1, 0), str1 * 0)

    def testAddedFunctionWithVarargs(self):
        '''Calls function that receives varargs added to module from type system description.'''
        self.assertEqual(sample.countVarargs(1), 0)
        self.assertEqual(sample.countVarargs(1, 2), 1)
        self.assertEqual(sample.countVarargs(1, 2, 3, 'a', 'b', 4, (5, 6)), 6)

    def testSampleComparisonOpInNamespace(self):
        s1 = sample.sample.sample(10)
        s2 = sample.sample.sample(10)
        self.assertEqual(s1, s2)

    def testConstant(self):
        self.assertEqual(sample.sample.INT_CONSTANT, 42)

    def testStringFunctions(self):
        # Test plain ASCII, UCS1 and UCS4 encoding which have different
        # representations in the PyUnicode objects.
        for t1 in ["ascii", "Ümläut", "😀"]:
            expected = t1 + t1
            self.assertEqual(sample.addStdStrings(t1, t1), expected)
            self.assertEqual(sample.addStdWStrings(t1, t1), expected)

    def testNullPtrT(self):
        sample.testNullPtrT(None)
        self.assertRaises(TypeError, sample.testNullPtrT, 42)

    def testRValueRefsWithValueTypes(self):
        """Passing value types by rvalue refs: For value types, nothing should
           happen since the argument is copied in the call and the copy is
           moved from."""
        polygon = sample.Polygon()
        polygon.addPoint(sample.Point(1, 2))
        polygon.addPoint(sample.Point(3, 4))
        point_count = len(polygon.points())
        self.assertEqual(point_count, sample.takePolygon(polygon))

    def testRValueRefsWithObjectTypes(self):
        """Passing object types by rvalue refs: The underlying object should
           be moved from."""
        o = sample.ObjectType()
        object_name = "Name"
        o.setObjectName(object_name)
        self.assertEqual(len(object_name), sample.takeObjectType(o))
        # o should be moved from, name is now empty
        self.assertEqual(len(o.objectName()), 0)

    def testMoveOnlyTypes(self):
        """Pass a move only type (convert using std::move())."""
        v = 42
        mo = sample.MoveOnly(v)
        mo2 = sample.MoveOnlyHandler.passMoveOnly(mo)
        self.assertEqual(mo2.value(), v)

    def testOptionalLong(self):
        v1 = 2
        v2 = 3
        self.assertEqual(sample.SampleNamespace.optionalMultiply(v1, v2), 6)
        self.assertIsNone(sample.SampleNamespace.optionalMultiply(v1, None))
        self.assertIsNone(sample.SampleNamespace.optionalMultiply(None, v2))

    def testSequenceProtocol(self):
        """Test slot Py_sq_contains of Polygon."""
        point1 = sample.Point(1, 2)
        point2 = sample.Point(3, 4)
        polygon = sample.Polygon()
        polygon.addPoint(point1)
        polygon.addPoint(point2)
        self.assertTrue(point1 in polygon)
        self.assertFalse(sample.Point(5, 6) in polygon)


if __name__ == '__main__':
    unittest.main()
